/*
	NOM PAQUET: PKG_CONTROL
	
	FUNCIONALITAT: Permet ACTIVAR i DESACTIVAR tant el LOG com el mode TEST. També proporciona les operaciones necessàries per saber
				   si el LOG està activat o no, aixi com també per registrar les diferents operacions en el LOG.

*/

spool 4_1_Implementacio_PkgControl.log;
-------------
--CAPÇALERA--
-------------
create or replace package pkg_control is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(11) := 'PKG_CONTROL';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	modeTest boolean;

	--Insereix dins de la taula de LOG_PROC_EXECUCIO (si el sistema el té ACTIVAT) el nom del procediment i els paràmetres corresponents a l'execució.
	--Retorna l'idLog corresponent.
	--Si el LOG no está ACTIVAT, retorna 0 i no fa res.
	FUNCTION REGISTRA_EXECUCIO  ( 
		procediment Varchar2,
		parametres Varchar2
	) RETURN INTEGER;
	
	--Per a un idLog concret insereix dins de la taula LOG_SEG_EXECUCIO la paraula "INICI". 
	PROCEDURE INICI_EXECUCIO  ( 
		idLog numeric
	);
	
	--Per a un idLog concret insereix dins de la taula LOG_SEG_EXECUCIO la paraula "FI".
	PROCEDURE FI_EXECUCIO  ( 
		idLog numeric
	);
	
	--Per a un idLog concret insereix dins de la taula LOG_SEG_EXECUCIO la paraula "TEST".
	--S'executa quan s'ha de fer una modificació i el sistema es troba en mode TEST
	PROCEDURE TEST_EXECUCIO  ( 
		idLog numeric
	);
	
	--Per a un idLog concret insereix dins de la taula LOG_SEG_EXECUCIO la paraula "ERROR" seguida d'un missatge d'error.
	--S'executa quan s'ha produït una excepció en el sistema i registra l'error.
	PROCEDURE ERROR_EXECUCIO  ( 
		idLog numeric,
		error Varchar2
	);
	
	--Comprova el valor de LOG_ACTIVAT a la taula CONFIGURACIO_SISTEMA. Si és 'S', retorna TRUE, si és 'N' retorna FALSE.
	FUNCTION LOG_ACTIVAT RETURN BOOLEAN;
	
	--Comprova el valor de MODE_TEST a la taula CONFIGURACIO_SISTEMA. Si és 'S', retorna TRUE, si és 'N' retorna FALSE.
	FUNCTION MODE_TEST RETURN BOOLEAN;
	
	--Modifica el valor de LOG_ACTIVAT, posant-lo a 'S' 
	PROCEDURE ACTIVA_LOG;
	
	--Modifica el valor de LOG_ACTIVAT, posant-lo a 'N'
	PROCEDURE DESACTIVA_LOG;
	
	--Modifica el valor de MODE_TEST, posant-lo a 'S'
	PROCEDURE ACTIVA_TEST;
	
	--Modifica el valor de MODE_TEST, posant-lo a 'N'
	PROCEDURE DESACTIVA_TEST;
	
	--Retorna el detall de les operacions realitzades per un idLog concret
	PROCEDURE LOG_ID (PidLog numeric);
	
	--Retorna el detall de les operacions realitzades en un període de temps concret
	PROCEDURE LOG_RANG_DATA (PdataInici IN DATE, PdataFi IN DATE);
	
	--Retorna totes les operacions que contenen un text concret
	PROCEDURE LOG_CERCA_OPERACIO (Ptext Varchar2);
end pkg_control;
/


--------
--BODY--
--------
create or replace package body pkg_control is

	---------------------
	--REGISTRA_EXECUCIO--
	---------------------
	FUNCTION REGISTRA_EXECUCIO  ( 
		procediment Varchar2,
		parametres Varchar2
	) RETURN INTEGER IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
		--LOG_ACTIU s'utilitza per determinar si la BD té el LOG activat o no.
		LOG_ACTIU BOOLEAN;
	BEGIN
		--Comprovam que el LOG està activat
		LOG_ACTIU:=pkg_Control.LOG_ACTIVAT();
		IF LOG_ACTIU THEN
			--Si el LOG està ACTIU, agafam de le seqûència un nou nombre
			idLog := seq_log.nextval;
			--Inserim dins de la taula LOG_PROC_EXECUCIO l'identificador de Log, el nom del procediment i els paràmetres amb els quals s'ha executat
			INSERT INTO LOG_PROC_EXECUCIO VALUES(idLog,procediment,parametres);
			
			--Registram el INICI de l'execució en la Taula de LOG_SEG_EXECUCIO (si el sistema el té activat)
			pkg_Control.INICI_EXECUCIO(idLog);
			COMMIT;
			--Retornam el idLog corresponent
			RETURN idLog;
		ELSE 
			--El LOG no està ACTIU i es retorna 0
			RETURN 0;
		END IF;
	END;
	
	
	------------------
	--INICI_EXECUCIO--
	------------------
	PROCEDURE INICI_EXECUCIO  ( 
		idLog numeric
		) IS
	BEGIN
		--Comprovam que el LOG està activat
		IF idLog > 0 THEN
			--Si el idLog és més gran de 0, el LOG està ACTIVAT i, per tant, inserim el idLog, la data actual i 'INICI' dins de la taula LOG_SEG_EXECUCIO
			INSERT INTO LOG_SEG_EXECUCIO VALUES(idLog,sysdate,'INICI');
			COMMIT;
		END IF;
	END;
	
	
	---------------
	--FI_EXECUCIO--
	---------------
	PROCEDURE FI_EXECUCIO  ( 
		idLog numeric
		) IS
	BEGIN
		--Comprovam que el LOG està activat
		IF idLog > 0 THEN
			--Si el idLog és més gran de 0, el LOG està ACTIVAT i, per tant, inserim el idLog, la data actual i 'FI' dins de la taula LOG_SEG_EXECUCIO
			INSERT INTO LOG_SEG_EXECUCIO VALUES(idLog,sysdate,'FI');
			COMMIT;
		END IF;
	END;
	
	
	-----------------
	--TEST_EXECUCIO--
	-----------------
	PROCEDURE TEST_EXECUCIO  ( 
		idLog numeric
	)IS
	BEGIN
		--Comprovam que el LOG està activat
		IF idLog > 0 THEN
			--Si el idLog és més gran de 0, el LOG està ACTIVAT i, per tant, inserim el idLog, la data actual i 'TEST' dins de la taula LOG_SEG_EXECUCIO
			INSERT INTO LOG_SEG_EXECUCIO VALUES(idLog,sysdate,'TEST');
			COMMIT;
		END IF;
	END;
	
	
	------------------
	--ERROR_EXECUCIO--
	------------------
	PROCEDURE ERROR_EXECUCIO  ( 
		idLog numeric,
		error Varchar2
		) IS
	BEGIN
		--Comprovam que el LOG està activat
		IF idLog > 0 THEN
			--Si el idLog és més gran de 0, el LOG està ACTIVAT i, per tant, inserim el idLog, la data actual 
			--i el missatge d'error dins de la taula LOG_SEG_EXECUCIO
			INSERT INTO LOG_SEG_EXECUCIO VALUES(idLog,sysdate,error);
			COMMIT;
		END IF;
	END;
	
	
	---------------
	--LOG_ACTIVAT--
	---------------	
	FUNCTION LOG_ACTIVAT RETURN BOOLEAN IS
		LA Varchar2(1);
	BEGIN
		--Carregam dins de LA el valor del paràmetre LOG_ACTIVAT de la taula CONFIGURACIO_SISTEMA
		SELECT valor INTO LA FROM configuracio_sistema WHERE parametre='LOG_ACTIVAT';
		IF LA='S' THEN
			--Si el valor de LA és 'S', el LOG està activat i retornam TRUE
			RETURN TRUE;
		ELSE
			--Si el valor de LA és 'N', el LOG està desactivat i retornam FALSE
			RETURN FALSE;
		END IF;
	END;
	
	
	--------------
	--ACTIVA_LOG--
	--------------	
	PROCEDURE ACTIVA_LOG IS
	BEGIN
		--Actualitzam el valor del parámetre LOG_ACTIVAT dins de la taula CONFIGURACIO_SISTEMA perquè sigui 'S'
		UPDATE CONFIGURACIO_SISTEMA SET VALOR='S' WHERE PARAMETRE='LOG_ACTIVAT';
		COMMIT;
	END;
	
	
	-----------------
	--DESACTIVA_LOG--
	-----------------	
	PROCEDURE DESACTIVA_LOG IS
	BEGIN
		--Actualitzam el valor del parámetre LOG_ACTIVAT dins de la taula CONFIGURACIO_SISTEMA perquè sigui 'N'
		UPDATE CONFIGURACIO_SISTEMA SET VALOR='N' WHERE PARAMETRE='LOG_ACTIVAT';
		COMMIT;
	END;
	
	
	-------------
	--MODE_TEST--
	-------------	
	FUNCTION MODE_TEST RETURN BOOLEAN IS
		MT Varchar2(1);
	BEGIN
		--Carregam dins de MT el valor del paràmetre MODE_TEST de la taula CONFIGURACIO_SISTEMA
		SELECT valor INTO MT FROM configuracio_sistema WHERE parametre='MODE_TEST';
		IF MT='S' THEN
			--Si el valor de MT és 'S', el TEST està activat i retornam TRUE
			RETURN TRUE;
		ELSE
			--Si el valor de MT és 'n', el TEST està desactivat i retornam FALSE
			RETURN FALSE;
		END IF;
	END;
	
	
	---------------
	--ACTIVA_TEST--
	---------------	
	PROCEDURE ACTIVA_TEST IS
	BEGIN
		--Actualitzam el valor del parámetre MODE_TEST dins de la taula CONFIGURACIO_SISTEMA perquè sigui 'S'
		UPDATE CONFIGURACIO_SISTEMA SET VALOR='S' WHERE PARAMETRE='MODE_TEST';
		COMMIT;
	END;
	
	
	------------------
	--DESACTIVA_TEST--
	------------------	
	PROCEDURE DESACTIVA_TEST IS
	BEGIN
		--Actualitzam el valor del parámetre MODE_TEST dins de la taula CONFIGURACIO_SISTEMA perquè sigui 'N'
		UPDATE CONFIGURACIO_SISTEMA SET VALOR='N' WHERE PARAMETRE='MODE_TEST';
		COMMIT;
	END;
	
	----------
	--LOG_ID--
	----------
	PROCEDURE LOG_ID (PidLog numeric) IS
		--Cursor que s'utilitza per agafar el nom i els paràmetres d'execució del procediment
		CURSOR CL1 IS SELECT PROCEDIMENTLOG, PARAMETRESLOG FROM LOG_PROC_EXECUCIO WHERE idLog=PidLog;
		--Cursor amb les operacions 
		CURSOR CL2 IS
			SELECT OPERACIO, DATALOG
			FROM LOG_SEG_EXECUCIO LS
			WHERE IDLOG=PidLog
			ORDER BY 2 ASC;
			
		TnomProcediment Varchar2(100);
		TParametres Varchar2(100);
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LOG_ID','P1:'||PidLog);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;								
		--Obrim el cursor
		OPEN CL1;
		FETCH CL1 INTO TnomProcediment, TParametres;
		--Revisam si ha retornat registres
		IF CL1%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL LOG AMB EL ID '||PidLog);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('PROCEDIMENT LOG: '||TnomProcediment);
			DBMS_OUTPUT.PUT_LINE('PARAMETRES LOG: '||TParametres);
			FOR REC IN CL2
			LOOP
				DBMS_OUTPUT.PUT_LINE('OPERACIO: '||REC.OPERACIO|| chr(9)||'  DATA: '||REC.DATALOG);
			END LOOP;
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL LOG AMB EL ID '||PidLog);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CL1;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	-----------------
	--LOG_RANG_DATA--
	-----------------
	PROCEDURE LOG_RANG_DATA (PdataInici IN DATE, PdataFi IN DATE) IS
		--Cursor que s'utilitza per determinar els logs que s'han generat en un rang concret de dates
		CURSOR RPC IS
			SELECT DISTINCT LS.IDLOG
			FROM LOG_PROC_EXECUCIO LP, LOG_SEG_EXECUCIO LS
			WHERE DATALOG BETWEEN PdataInici AND PdataFi
			AND LS.IDLOG=LP.IDLOG
			ORDER BY 1 ASC;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN				
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LOG_RANG_DATA','P1:'||PdataInici||' P2:'||PdataFi);	
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		--Imprimim les dades de les revisions
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;													
		FOR rpc_rec IN RPC
		LOOP
			PKG_CONTROL.LOG_ID(rpc_rec.idLog);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------------
	--LOG_CERCA_OPERACIO--
	----------------------
	PROCEDURE LOG_CERCA_OPERACIO (Ptext Varchar2) IS
		--Cursor que s'utilitza per determinar els logs que s'han generat en un rang concret de dates
		CURSOR RPC IS
			SELECT DISTINCT IDLOG
			FROM LOG_SEG_EXECUCIO LS
			WHERE OPERACIO LIKE '%'||Ptext||'%'; 
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN				
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LOG_CERCA_OPERACIO','P1:'||Ptext);	
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		--Imprimim les dades de les revisions
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;													
		FOR rpc_rec IN RPC
		LOOP
			PKG_CONTROL.LOG_ID(rpc_rec.idLog);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
END pkg_control;
/

spool off;
