/*
	NOM PAQUET: PKG_CLIENT
	
	FUNCIONALITAT: Proporciona les operacions necessàries per treballar amb els clients

*/

spool 4_2_Implementacio_PkgClient.log;
-------------
--CAPÇALERA--
-------------
create or replace package pkg_CLIENT is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(10) := 'PKG_CLIENT';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	
	--Permet donar d'alta un nou client en el sistema.
	PROCEDURE ALTA_CLIENT (
		PidClient numeric,
		PnomClient Varchar2,
		Pllinatge1Client Varchar2,
		Pllinatge2Client Varchar2,
		PdataNaixementClient date,
		PsexeClient Varchar2, --E (Empresa), M (Masculí), F (Femení)
		PdniCifClient Varchar2,
		PtelefonClient numeric,
		PdataAlta date
	);
	
	--Permet donar de baixa un client en el sistema
	PROCEDURE BAIXA_CLIENT ( 
		PidClient numeric
	);
	
	--Permet modificar les dades d'un client existent
	PROCEDURE MODIFICA_CLIENT (
		PidClient numeric,
		PnomClient Varchar2,
		Pllinatge1Client Varchar2,
		Pllinatge2Client Varchar2,
		PdataNaixementClient date,
		PsexeClient Varchar2, --E (Empresa), M (Masculí), F (Femení)
		PdniCifClient Varchar2,
		PtelefonClient numeric,
		PdataAlta date
	);
	
end pkg_CLIENT;
/

--------
--BODY--
--------
create or replace package body pkg_CLIENT is

	---------------
	--ALTA_CLIENT--
	---------------
	PROCEDURE ALTA_CLIENT (
		PidClient numeric,
		PnomClient Varchar2,
		Pllinatge1Client Varchar2,
		Pllinatge2Client Varchar2,
		PdataNaixementClient date,
		PsexeClient Varchar2, --E (Empresa), M (Masculí), F (Femení)
		PdniCifClient Varchar2,
		PtelefonClient numeric,
		PdataAlta date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_CLIENT','P1:'||PidClient||' P2:'||PnomClient||' P3:'||Pllinatge1Client||
									     ' P4:'||Pllinatge2Client||' P5:'||PdataNaixementClient||' P6:'||PsexeClient||' P7:'||PtelefonClient||' P8:'||PdataAlta||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidClient = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou client en el sistema
				INSERT INTO CLIENT VALUES(seq_client.nextval,PnomClient,Pllinatge1Client,Pllinatge2Client,PdataNaixementClient,PsexeClient,PdniCifClient,PtelefonClient,PdataAlta);
			ELSE
				--Si l'identificador no és igual a 0, inserim el client amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO CLIENT VALUES(PidClient,PnomClient,Pllinatge1Client,Pllinatge2Client,PdataNaixementClient,PsexeClient,PdniCifClient,PtelefonClient,PdataAlta);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------
	--BAIXA_CLIENT--
	----------------
	PROCEDURE BAIXA_CLIENT ( 
		PidClient numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_CLIENT','P1:'||PidClient||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el client i confirmam el canvi
			DELETE FROM CLIENT WHERE idClient=PidClient;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-------------------
	--MODIFICA_CLIENT--
	-------------------
	PROCEDURE MODIFICA_CLIENT (
		PidClient numeric,
		PnomClient Varchar2,
		Pllinatge1Client Varchar2,
		Pllinatge2Client Varchar2,
		PdataNaixementClient date,
		PsexeClient Varchar2, --E (Empresa), M (Masculí), F (Femení)
		PdniCifClient Varchar2,
		PtelefonClient numeric,
		PdataAlta date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_CLIENT','P1:'||PidClient||' P2:'||PnomClient||' P3:'||Pllinatge1Client||
									     ' P4:'||Pllinatge2Client||' P5:'||PdataNaixementClient||' P6:'||PsexeClient||' P7:'||PtelefonClient||' P8:'||PdataAlta||'');
									     
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE CLIENT SET nomClient=PnomClient,llinatge1Client=Pllinatge1Client,llinatge2Client=Pllinatge2Client,
				dataNaixementClient=PdataNaixementClient,dniCifClient=PdniCifClient,telefonClient=PtelefonClient,dataAlta=PdataAlta WHERE idClient=PidClient;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;

	
END;
/

spool off;
