/*
	NOM PAQUET: PKG_COTXES
	
	FUNCIONALITAT: Proporciona les operaciones necessàries per actualitzar, modificar i eliminar les MARQUES, els MODELS, els EXTRES i els COTXES del sistema

*/

spool 4_3_Implementacio_PkgCotxes.log;
-------------
--CAPÇALERA--
-------------
create or replace package pkg_Cotxes is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(10) := 'PKG_COTXES';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	--Permet donar d'alta una nova marca en el sistema
	PROCEDURE ALTA_MARCA (
		PidMarca numeric,
		PnomMarca Varchar2
	);
	
	--Permet donar de baixa una marca en el sistema
	PROCEDURE BAIXA_MARCA ( 
		PidMarca numeric
	);
	
	--Permet modificar una marca en el sistema
	PROCEDURE MODIFICA_MARCA (
		PidMarca numeric,
		PnomMarca Varchar2
	);
	
	--Permet donar d'alta un model de cotxe en el sistema
	PROCEDURE ALTA_MODEL (
		PidModel numeric,
		PnomModel varchar2,
		PpreuModel number,
		PpotenciaModel number,
		PnumPortesModel number,
		PcilindradaModel number,
		PcombustibleModel varchar2,
		PidMarca numeric
	);
	
	--Permet donar de baixa un model de cotxe en el sistema
	PROCEDURE BAIXA_MODEL ( 
		PidModel numeric
	);
	
	--Permet modificar les dades d'un model de cotxe del sistema
	PROCEDURE MODIFICA_MODEL (
		PidModel numeric,
		PnomModel varchar2,
		PpreuModel number,
		PpotenciaModel number,
		PnumPortesModel number,
		PcilindradaModel number,
		PcombustibleModel varchar2,
		PidMarca numeric
	);
	
	--Permet donar d'alta un extra de cotxe en el sistema
	PROCEDURE ALTA_EXTRA (
		PidExtra numeric,
		PnomExtra Varchar2
	);
	
	--Permet donar de baixa un extra de cotxe en el sistema
	PROCEDURE BAIXA_EXTRA ( 
		PidExtra numeric
	);
	
	--Permet modificar el nom d'un extra de cotxe del sistema
	PROCEDURE MODIFICA_EXTRA (
		PidExtra numeric,
		PnomExtra Varchar2
	);
	
	--Permet associar un extra a un model amb un preu concret 
	PROCEDURE AFEGIR_EXTRA_MODEL ( 
		PidExtra numeric,
		PidModel numeric,
		PpreuExtra number,
		PserieOpcio Varchar2
	);
	
	--Permet desassociar un extra a un model amb un preu concret 
	PROCEDURE ELIMINAR_EXTRA_MODEL (
		PidExtra numeric,
		PidModel numeric
	);
	
	--Permet modificar un extra a un model amb un preu concret 
	PROCEDURE MODIFICAR_EXTRA_MODEL ( 
		PidExtra numeric,
		PidModel numeric,
		PpreuExtra number,
		PserieOpcio Varchar2
	);
	
	--Permet donar d'alta un cotxe en el sistema
	PROCEDURE ALTA_COTXE (
		PidCotxe numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2,
		PidModel numeric
	);
	
	--Permet donar de baixa un cotxe en el sistema
	PROCEDURE BAIXA_COTXE (
		PidCotxe numeric
	);
	
	--Permet modificar les dades d'un cotxe del sistema
	PROCEDURE MODIFICA_COTXE (
		PidCotxe numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2,
		PidModel numeric
	);	
	
	
end pkg_COTXES;
/


--------
--BODY--
--------
create or replace package body pkg_COTXES is

	--------------
	--ALTA_MARCA--
	--------------
	PROCEDURE ALTA_MARCA (
		PidMarca numeric,
		PnomMarca Varchar2
	)	IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_MARCA','P1:'||PidMarca||' P2:'||PnomMarca||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidMarca = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim la nova marca al sistema
				INSERT INTO MARCA VALUES(seq_marca.nextval,PnomMarca);
			ELSE
				--Si l'identificador no és igual a 0, inserim la marca amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO MARCA VALUES(PidMarca,PnomMarca);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------
	--BAIXA_MARCA--
	---------------
	PROCEDURE BAIXA_MARCA ( 
		PidMarca numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_MARCA','P1:'||PidMarca||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la marca del sistema i confirmam el canvi
			DELETE FROM MARCA WHERE idMarca=PidMarca;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------
	--MODIFICA_MARCA--
	------------------
	PROCEDURE MODIFICA_MARCA (
		PidMarca numeric,
		PnomMarca Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_MARCA','P1:'||PidMarca||' P2:'||PnomMarca||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE MARCA SET nomMarca=PnomMarca WHERE idMarca=PidMarca;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat) 
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------
	--ALTA_MODEL--
	--------------
	PROCEDURE ALTA_MODEL (
		PidModel numeric,
		PnomModel varchar2,
		PpreuModel number,
		PpotenciaModel number,
		PnumPortesModel number,
		PcilindradaModel number,
		PcombustibleModel varchar2,
		PidMarca numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_MODEL','P1:'||PidModel||' P2:'||PnomModel||' P3:'||PpreuModel||' P4:'||PpotenciaModel||
										 ' P5:'||PnumPortesModel||' P6:'||PcilindradaModel||' P7:'||PcombustibleModel||' P8:'||PidMarca||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidModel = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou model al sistema
				INSERT INTO MODEL VALUES(seq_model.nextval,PnomModel,PpreuModel,PpotenciaModel,PnumPortesModel,PcilindradaModel,PcombustibleModel,PidMarca);
			ELSE
				--Si l'identificador no és igual a 0, inserim el model amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO MODEL VALUES(PidModel,PnomModel,PpreuModel,PpotenciaModel,PnumPortesModel,PcilindradaModel,PcombustibleModel,PidMarca);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------
	--BAIXA_MODEL--
	---------------
	PROCEDURE BAIXA_MODEL ( 
		PidModel numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_MODEL','P1:'||PidModel||'');
		
		--Comprovam si el sistema es troba en mode TEST			
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el model i confirmam el canvi
			DELETE FROM MODEL WHERE idModel=PidModel;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------
	--MODIFICA_MODEL--
	------------------
	PROCEDURE MODIFICA_MODEL (
		PidModel numeric,
		PnomModel varchar2,
		PpreuModel number,
		PpotenciaModel number,
		PnumPortesModel number,
		PcilindradaModel number,
		PcombustibleModel varchar2,
		PidMarca numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_MODEL','P1:'||PidModel||' P2:'||PnomModel||' P3:'||PpreuModel||' P4:'||PpotenciaModel||
										 ' P5:'||PnumPortesModel||' P6:'||PcilindradaModel||' P7:'||PcombustibleModel||' P8:'||PidMarca||'');
		
		--Comprovam si el sistema es troba en mode TEST			
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE MODEL SET preuModel=PpreuModel,nomModel=PnomModel,potenciaModel=PpotenciaModel,numPortesModel=PnumPortesModel,
				cilindradaModel=PcilindradaModel,combustibleModel=PcombustibleModel,idMarca=PidMarca WHERE idModel=PidModel;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;

	
	--------------
	--ALTA_EXTRA--
	--------------
	PROCEDURE ALTA_EXTRA (
		PidExtra numeric,
		PnomExtra Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_EXTRA','P1:'||PidExtra||' P2:'||PnomExtra||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidExtra = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou extra al sistema
				INSERT INTO EXTRA VALUES(seq_extra.nextval,PnomExtra);
			ELSE
				--Si l'identificador no és igual a 0, inserim l'extra amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO EXTRA VALUES(PidExtra,PnomExtra);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------
	--BAIXA_EXTRA--
	---------------
	PROCEDURE BAIXA_EXTRA ( 
		PidExtra numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_EXTRA','P1:'||PidExtra||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram l'extra i confirmam el canvi
			DELETE FROM EXTRA WHERE idExtra=PidExtra;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------
	--MODIFICA_EXTRA--
	------------------	
	PROCEDURE MODIFICA_EXTRA (
		PidExtra numeric,
		PnomExtra Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_EXTRA','P1:'||PidExtra||' P2:'||PnomExtra||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE EXTRA SET nomExtra=PnomExtra WHERE idExtra=PidExtra;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------------
	--AFEGIR_EXTRA_MODEL--
	----------------------	
	PROCEDURE AFEGIR_EXTRA_MODEL (
		PidExtra numeric,
		PidModel numeric,
		PpreuExtra number,
		PserieOpcio Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.AFEGIR_EXTRA_MODEL_SERIE','P1:'||PidExtra||' P2:'||PidModel||' P3:'||PpreuExtra||
												    ' P4:'||pserieOpcio||'');
												    
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, inserim la relació entre un extra i el model
			INSERT INTO EXTRAMODEL VALUES(PidExtra,PidModel,PpreuExtra,PserieOpcio);
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------------
	--ELIMINAR_EXTRA_MODEL--
	------------------------	
	PROCEDURE ELIMINAR_EXTRA_MODEL ( 
		PidExtra numeric,
		PidModel numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ELIMINAR_EXTRA_MODEL_SERIE','P1:'||PidExtra||' P2:'||PidModel||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la relació entre un extra i un model
			DELETE FROM EXTRAMODEL WHERE idExtra=PidExtra and idModel=PidModel;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-------------------------
	--MODIFICAR_EXTRA_MODEL--
	-------------------------	
	PROCEDURE MODIFICAR_EXTRA_MODEL ( 
		PidExtra numeric,
		PidModel numeric,
		PpreuExtra number,
		PserieOpcio Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICAR_EXTRA_MODEL_SERIE','P1:'||PidExtra||' P2:'||PidModel||' P3:'||PpreuExtra||
												    ' P4:'||PserieOpcio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, modificam els paràmetres de la relació entre un extra i un model
			UPDATE EXTRAMODEL SET preuExtra=PpreuExtra,serieOpcio=PserieOpcio WHERE idExtra=PidExtra and idModel=PidModel;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------
	--ALTA_COTXE--
	--------------	
	PROCEDURE ALTA_COTXE (
		PidCotxe numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2,
		PidModel numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_COTXE','P1:'||PidCotxe||' P2:'||PbastidorCotxe||' P3:'||PmatriculaCotxe||' P4:'||PidModel||'');

		--Comprovam si el sistema es troba en mode TEST				
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidCotxe = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou cotxe al sistema
				INSERT INTO COTXE VALUES(seq_cotxe.nextval,PbastidorCotxe,PmatriculaCotxe,PidModel);
			ELSE
				--Si l'identificador no és igual a 0, inserim el cotxe amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO COTXE VALUES(PidCotxe,PbastidorCotxe,PmatriculaCotxe,PidModel);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------
	--BAIXA_COTXE--
	---------------	
	PROCEDURE BAIXA_COTXE (
		PidCotxe numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_COTXE','P1:'||PidCotxe||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el cotxe i confirmam el canvi
			DELETE FROM COTXE WHERE idCotxe=PidCotxe;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------
	--MODIFICA_COTXE--
	------------------	
	PROCEDURE MODIFICA_COTXE (
		PidCotxe numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2,
		PidModel numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_COTXE','P1:'||PidCotxe||' P2:'||PbastidorCotxe||' P3:'||PmatriculaCotxe||' P4:'||PidModel||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE COTXE SET bastidorCotxe=PbastidorCotxe,matriculaCotxe=PmatriculaCotxe,idModel=PidModel WHERE idCotxe=PidCotxe;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	

END;
/

spool off;
