/*
	NOM PAQUET: PKG_DWH
	
	FUNCIONALITAT: Proporciona les operacions necessàries per carregar les diferents dimensions i fets del DWH, 
	               així com també les funcions que permetin consultar fàcilment els ranquings demanats pel client.
	               

*/


spool 4_4_Implementacio_PkgDWH.log
-------------
--CAPÇALERA--
-------------
create or replace package pkg_DWH is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(7) := 'PKG_DWH';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	--Tipus RECORD i TABLE que permeten que les funcions de consultes de ranquings i agrupats retornin taules
	TYPE RANQUING_REC IS RECORD (
		POSICIO number,
		IDENTIFICADOR number,
		QUANTITAT number
	);
	
	TYPE AGRUPAT_REC IS RECORD (
		NUMANY number,
		NUMMES number,
		QUANTITAT NUMBER
	);
	
	--Inicialitza la dimensió data dins el període que se li passa per paràmetre	
	PROCEDURE INICIALITZA_DIMENSIO_DATA(PdataInici IN DATE, PdataFi IN DATE);
	
	--Actualitza les dades de la taula CONTROL_DWH, per així saber quan s'ha actualitzat per darrera vegada tant les dimensions com els fets
	PROCEDURE ACTUALITZA_CONTROL_DWH(PProcediment IN Varchar2, PdataInici IN DATE, PdataFi IN DATE);
	
	--Insereix un client (i el seu sexe) dins de la taula de dimensions DCLIENT
	PROCEDURE CARREGA_CLIENT (idClient IN NUMBER, sexeClient IN NUMBER);
	
	--Per a un rang de dates, insereix tots els clients que s'hagin donat d'alta en aquell període
	PROCEDURE CARREGA_DIMENSIO_CLIENT(PdataInici IN DATE, PdataFi IN DATE);
	
	--Insereix un venedor dins de la taula de dimensions DVENEDOR
	PROCEDURE CARREGA_VENEDOR (idVenedor IN NUMBER);
	
	--Per a un rang de dates, insereix tots els venedors que s'hagin donat d'alta en aquell període
	PROCEDURE CARREGA_DIMENSIO_VENEDOR(PdataInici IN DATE, PdataFi IN DATE);
		
	--Per a un rang de dates, carrega les dades relacionades amb el temps mitjà de reparacio
	PROCEDURE CARREGA_FETS_TEMPSREPARACIO(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega el nombre de reparacions per dia realitzades
	PROCEDURE CARREGA_FETS_AGRUPAREPARACIO(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega el nombre der revisions per dia realitzades
	PROCEDURE CARREGA_FETS_AGRUPAREVISIO(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega el nombre de vendes per dia realitzades
	PROCEDURE CARREGA_FETS_AGRUPAVENDA(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega per client la suma de la quantitat gastada per dia 
	PROCEDURE CARREGA_FETS_DESPESACLIENT(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega per venedor la suma de les vendes realitzades per dia
	PROCEDURE CARREGA_FETS_RANQUINGVENEDOR(PdataInici IN DATE, PdataFi IN DATE);
	
	--Per a un rang de dates, carrega totes les dades relacionades amb les dimensions CLIENT i VENEDOR 
	--i els fets TEMPSREPARACIOCOTXE, MESOSREPARACIO, MESOSREVISIO, MESOSVENDA, DESPESACLIENT i RANQUINGVENEDOR
	PROCEDURE ACTUALITZA_DWH(PdataInici IN DATE, PdataFi IN DATE);
	
	--Motra per pantalla la mitjana de temps de reparació de reparació dels vehicles
	PROCEDURE TEMPSMITJANAREPARACIO;
	
	--Per a any i un mes concret, retorna quin ha estat el venedor que ha realitzat més vendes
	PROCEDURE RANQUINGMESVENEDOR (PANY NUMERIC, PMES NUMERIC);
	
	--Per a un rang de mesos (amb el format YYYYMM) retorna el nombre de revisions realitzades per mes
	PROCEDURE RANQUINGMESREVISIO (PDATAINI NUMERIC, PDATAFI NUMERIC);
	
	--Per a un rang de mesos (amb el format YYYYMM) retorna el nombre de reparacions realitzades per mes
	PROCEDURE RANQUINGMESREPARACIO (PDATAINI NUMERIC, PDATAFI NUMERIC);
	
	--Per a un rang de mesos (amb el format YYYYMM) retorna el nombre de vendes realitzades per mes
	PROCEDURE RANQUINGMESVENDA (PDATAINI NUMERIC, PDATAFI NUMERIC);
	
	--Retorna el RANQUING de clients que han realitzat més despesa en tot el temps
	PROCEDURE RANQUINGDESPESACLIENT;
	
END PKG_DWH;
/

--------
--BODY--
--------
CREATE OR REPLACE PACKAGE BODY PKG_DWH IS

	-----------------------------
	--INICIALITZA_DIMENSIO_DATA--
	-----------------------------
	PROCEDURE INICIALITZA_DIMENSIO_DATA(PdataInici IN DATE, PdataFi IN DATE) IS
		VseqId number;
		VloadData date;
		NdataInici number;
		NdataFi number;
	
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.INICIALITZA_DIMENSIO_DATA','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de data concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('INICIALITZA_DIMENSIO_DATA',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Passam les dates a un nombre del tipus YYYYMMDD
			NdataInici := to_number(to_char(PdataInici,'yyyymmdd'));
			NdataFi := to_number(to_char(PdataFi,'yyyymmdd'));
		
			--Inicialitzam els índexos
			VseqId :=  NdataInici;
			VloadData:=PdataInici;

			--Inicialitzam la insersió iterativa
			LOOP
				--Inserim la seqüencia del tipus YYYYMMDD, l'any, el mes i el dia dins de la dimensió DDATA.
				INSERT INTO dData VALUES (
					VseqId,
					to_char(VloadData,'yyyy'),
					to_char(VloadData,'mm'),
					to_char(VloadData,'dd')
				);
		
				--Incrementam en un dia la data i actualitzam el identificador
				VloadData := VloadData + 1;
				VseqId:=to_number(to_char(VloadData,'yyyymmdd'));
		
				--Si l'identificador es superior a la data final pasada per paràmetre, finalitza la iteració
				IF VseqId>NdataFi THEN 
					EXIT;
				END IF;
			END LOOP;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END INICIALITZA_DIMENSIO_DATA;
	
	
	--------------------------
	--ACTUALITZA_CONTROL_DWH--
	--------------------------
	PROCEDURE ACTUALITZA_CONTROL_DWH(PProcediment IN Varchar2, PdataInici IN DATE, PdataFi IN DATE) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.INICIALITZA_DIMENSIO_DATA','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Comprovam si el sistema es troba en mode TEST		
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal i actualitzam el registre corresponent de la taula CONTROL_DWH
			UPDATE CONTROL_DWH SET data_inicial_darrera_execucio=PdataInici,data_final_darrera_execucio=PdataFi,data_execucio=SYSDATE
					WHERE procediment=PProcediment;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END ACTUALITZA_CONTROL_DWH;
	
	
	------------------
	--CARREGA_CLIENT--
	------------------
	PROCEDURE CARREGA_CLIENT (idClient IN NUMBER, sexeClient IN NUMBER) IS
			--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
			idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_CLIENT','P1:'||idClient||' P2:'||sexeClient||'');
		
		--Comprovam si el sistema es troba en mode TEST		
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, inserim el client corresponent dins de la dimensió DCLIENT
			INSERT INTO DCLIENT VALUES(idClient,sexeClient);
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_CLIENT;


	---------------------------
	--CARREGA_DIMENSIO_CLIENT--
	---------------------------
	PROCEDURE CARREGA_DIMENSIO_CLIENT(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor DC per extreure de la taula CLIENT els clients que han estat donats d'alta entre les dates passades per paràmetre
		CURSOR DC IS
			SELECT idCLIENT,sexeClient FROM CLIENT WHERE DATAALTA BETWEEN PdataInici AND PdataFi;	
			
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_DIMENSIO_CLIENT','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_DIMENSIO_CLIENT',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--TRANSFORMACIO I CARREGA DE DADES--
			FOR dc_rec IN DC
			LOOP
				--Per a cada client, transformam el SEXE a un nombre
				IF dc_rec.sexeClient = 'M' THEN
					--0 per a M
					PKG_DWH.CARREGA_CLIENT(dc_rec.idClient,0);
				ELSIF dc_rec.sexeClient = 'F' THEN
					--1 per a F
					PKG_DWH.CARREGA_CLIENT(dc_rec.idClient,1);
				ELSIF  dc_rec.sexeClient = 'E' THEN
					--2 per a E
					PKG_DWH.CARREGA_CLIENT(dc_rec.idClient,2);
				ELSE
					--3 si no és cap dels anteriors
					PKG_DWH.CARREGA_CLIENT(dc_rec.idClient,3);
				END IF;
			END LOOP;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_DIMENSIO_CLIENT;


	-------------------
	--CARREGA_VENEDOR--
	-------------------	
	PROCEDURE CARREGA_VENEDOR (idVenedor IN NUMBER) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució	
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_VENEDOR','P1:'||idVenedor||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, inserim el nou venedor dins de la dimensió DVENEDOR
			INSERT INTO DVENEDOR VALUES(idVenedor);
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_VENEDOR;


	----------------------------
	--CARREGA_DIMENSIO_VENEDOR--
	----------------------------
	PROCEDURE CARREGA_DIMENSIO_VENEDOR(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor VC per extreure de la taula VENEDOR els venedors que han estat donats d'alta entre les dates passades per paràmetre
		CURSOR VC IS
			SELECT idVenedor FROM VENEDOR WHERE DATAALTA BETWEEN PdataInici AND PdataFi;
	
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_DIMENSIO_VENEDOR','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_DIMENSIO_VENEDOR',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--CARREGA DE DADES--
			FOR vc_rec IN VC
			LOOP
				PKG_DWH.CARREGA_VENEDOR(vc_rec.idVenedor);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_DIMENSIO_VENEDOR;
	
	
	-------------------------------
	--CARREGA_FETS_TEMPSREPARACIO--
	-------------------------------
	PROCEDURE CARREGA_FETS_TEMPSREPARACIO(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules REPARACIO i HAREPARAT les dades que tenen com a data el període passat per paràmetre	
		CURSOR RPC IS
			SELECT trunc(R.dataFiReparacio) dFR,SUM(H.numHoresReparacio) horesReparacio,COUNT(DISTINCT R.idReparacio) numReparacions
				FROM REPARACIO R, HAREPARAT H 
				WHERE R.idReparacio=H.idReparacio
				AND R.dataFiReparacio is not null
				AND dataFiReparacio BETWEEN PdataInici AND PdataFi
				GROUP BY trunc(R.dataFiReparacio)
				ORDER BY 1;
				
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_TEMPSREPARACIO','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_TEMPSREPARACIO',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO TEMPSREPARACIOCOTXE VALUES(to_number(to_char(rpc_rec.dFR,'yyyymmdd')),rpc_rec.horesReparacio/rpc_rec.numReparacions);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_TEMPSREPARACIO;
	
	
	--------------------------------
	--CARREGA_FETS_AGRUPAREPARACIO--
	--------------------------------
	PROCEDURE CARREGA_FETS_AGRUPAREPARACIO(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules REPARACIO les dades que tenen com a data el període passat per paràmetre	
		CURSOR RPC IS
			SELECT trunc(R.dataFiReparacio) dFR, COUNT(DISTINCT R.idReparacio) numReparacions
				FROM REPARACIO R
				WHERE R.dataFiReparacio is not null
				AND dataFiReparacio BETWEEN PdataInici AND PdataFi
				GROUP BY trunc(R.dataFiReparacio)
				ORDER BY 1;
				
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_AGRUPAREPARACIO','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_AGRUPAREPARACIO',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO AGRUPATREPARACIO VALUES(to_number(to_char(rpc_rec.dFR,'yyyymmdd')),rpc_rec.numReparacions);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_AGRUPAREPARACIO;
	
	
	------------------------------
	--CARREGA_FETS_AGRUPAREVISIO--
	------------------------------
	PROCEDURE CARREGA_FETS_AGRUPAREVISIO(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules REVISIO les dades que tenen com a data el període passat per paràmetre			
		CURSOR RPC IS
			SELECT trunc(R.dataRevisio) dFR, COUNT(DISTINCT R.idRevisio) numRevisions
				FROM REVISIO R
				WHERE R.dataRevisio is not null
				AND dataRevisio BETWEEN PdataInici AND PdataFi
				GROUP BY trunc(R.dataRevisio)
				ORDER BY 1;
	
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_AGRUPAREVISIO','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_AGRUPAREVISIO',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO AGRUPATREVISIO VALUES(to_number(to_char(rpc_rec.dFR,'yyyymmdd')),rpc_rec.numRevisions);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_AGRUPAREVISIO;
	
	
	----------------------------
	--CARREGA_FETS_AGRUPAVENDA--
	----------------------------
	PROCEDURE CARREGA_FETS_AGRUPAVENDA(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules VENDA les dades que tenen com a data el període passat per paràmetre					
		CURSOR RPC IS
			SELECT trunc(V.dataVenda) dV, COUNT(DISTINCT V.idVenda) numVendes
				FROM VENDA V
				WHERE V.dataVenda is not null
				AND dataVenda BETWEEN PdataInici AND PdataFi
				GROUP BY trunc(V.dataVenda)
				ORDER BY 1;
				
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_AGRUPAVENDA','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_AGRUPAVENDA',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO AGRUPATVENDA VALUES(to_number(to_char(rpc_rec.dV,'yyyymmdd')),rpc_rec.numVendes);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_AGRUPAVENDA;
	
	
	------------------------------
	--CARREGA_FETS_DESPESACLIENT--
	------------------------------
	PROCEDURE CARREGA_FETS_DESPESACLIENT(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
		SELECT idClient,trunc(data) dataDespesa, sum(cost) despesa FROM (
			SELECT idClient,trunc(dataRevisio) data,SUM(numHoresRevisio*preuHoresRevisio) cost FROM REVISIO RV, COTXESTALLER CT, HAREVISAT HRV
				WHERE RV.idCotxe=CT.idCotxe
				AND HRV.idRevisio=RV.idRevisio
				AND dataRevisio IS NOT NULL
				AND esPressupost='N'
				AND dataRevisio BETWEEN PdataInici AND PdataFi
				GROUP BY idClient,trunc(dataRevisio)
			UNION
			SELECT idClient,trunc(dataFiReparacio) data,SUM(numHoresReparacio*preuHoresReparacio)+costPecesReparacio cost FROM REPARACIO RP, COTXESTALLER CT, HAREPARAT HRP
				WHERE Rp.idCotxe=CT.idCotxe
				AND HRP.idReparacio=RP.idReparacio
				AND dataFiReparacio IS NOT NULL
				AND esPressupost='N'
				AND dataFiReparacio BETWEEN PdataInici AND PdataFi
				GROUP BY idClient,trunc(dataFiReparacio),costPecesReparacio
			UNION
			SELECT idClient, trunc(V.dataVenda) data, SUM(preuVenda) cost
				FROM VENDA V
				WHERE V.dataVenda is not null
				AND dataVenda BETWEEN PdataInici AND PdataFi
				GROUP BY idClient,trunc(V.dataVenda)
		) GROUP BY idClient,data
		ORDER BY 1,2;
	
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_DESPESACLIENT','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_DESPESACLIENT',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO DESPESACLIENT VALUES(rpc_rec.idClient,to_number(to_char(rpc_rec.dataDespesa,'yyyymmdd')),rpc_rec.despesa);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_DESPESACLIENT;
	
	
	--------------------------------
	--CARREGA_FETS_RANQUINGVENEDOR--
	--------------------------------
	PROCEDURE CARREGA_FETS_RANQUINGVENEDOR(PdataInici IN DATE, PdataFi IN DATE) IS
		--EXTRACCIO DE DADES--
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
		select idVenedor,trunc(dataVenda) dataVenda,sum(preuVenda) preuVenda FROM VENDA
		GROUP BY idVenedor,trunc(dataVenda)
		ORDER BY 1,2;
		
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CARREGA_FETS_RANQUINGVENEDOR','P1:'||PdataInici||' P2:'||PdataFi||'');
		
		--Registram dins la taula CONTROL_DWH que s'executará el procediment per a un rang de dates concret
		PKG_DWH.ACTUALITZA_CONTROL_DWH('CARREGA_FETS_RANQUINGVENEDOR',PdataInici,PdataFi);
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--TRACTAMENT I CARREGA DE DADES--
			--Si no es troba en mode Test, inserim dins de la taula de fets les dades transformades
			FOR rpc_rec IN RPC
			LOOP
				INSERT INTO RANQUINGVENEDOR VALUES(to_number(to_char(rpc_rec.dataVenda,'yyyymmdd')),rpc_rec.idVenedor,rpc_rec.preuVenda);
			END LOOP;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CARREGA_FETS_RANQUINGVENEDOR;
	
	
	------------------
	--ACTUALITZA_DWH--
	------------------
	PROCEDURE ACTUALITZA_DWH(PdataInici IN DATE, PdataFi IN DATE) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Carregam primer les dimensions CLIENT i VENEDOR (la dimensió data es carrega a part) del període passat per paràmetre
		CARREGA_DIMENSIO_CLIENT(PdataInici, PdataFi);
    
		CARREGA_DIMENSIO_VENEDOR(PdataInici,PdataFi);
		
		--Carregam tots els fets corresponents al període passat per paràmetre
		CARREGA_FETS_TEMPSREPARACIO(PdataInici, PdataFi);
	
		CARREGA_FETS_AGRUPAREPARACIO(PdataInici, PdataFi);
	
		CARREGA_FETS_AGRUPAREVISIO(PdataInici, PdataFi);
	
		CARREGA_FETS_AGRUPAVENDA(PdataInici, PdataFi);
	
		CARREGA_FETS_DESPESACLIENT(PdataInici, PdataFi);
	
		CARREGA_FETS_RANQUINGVENEDOR(PdataInici, PdataFi);
	END ACTUALITZA_DWH;
	
	
	-------------------------
	--TEMPSMITJANAREPARACIO--
	-------------------------
	PROCEDURE TEMPSMITJANAREPARACIO IS
		--Variable on desam la mitjana
		Tmitjana numeric(8,2);
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.TEMPSMITJANAREPARACIO','NO APLICA');
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		SELECT SUM(TEMPSMITJANAREPARACIO)/COUNT(1) INTO Tmitjana FROM TEMPSREPARACIOCOTXE;
		DBMS_OUTPUT.PUT_LINE('LA MITJANA DE REPARACIO D''UN COTXE ES: '||Tmitjana||' HORES');
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END TEMPSMITJANAREPARACIO;
	
	
	----------------------
	--RANQUINGMESVENEDOR--
	----------------------
	PROCEDURE RANQUINGMESVENEDOR (PANY NUMERIC, PMES NUMERIC) IS
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
			SELECT idVenedor,sum(venda) venda FROM ranquingVenedor rV, ddata d 
				WHERE d.idData=rV.idData and mesany=PMES and d.dataany=PANY
				group by idVenedor
				ORDER BY 2 DESC;
				
		--Utilitzam la variable POSICIO per poder controlar fàcilment quina posició ocupa dins del rànquing
		POSICIO INTEGER:=1;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.RANQUINGMESVENEDOR','P1:'||PANY||' P2:'||PMES);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('POSICIO '||POSICIO||chr(9)||'  IDVENEDOR: '|| rpc_rec.idVenedor||chr(9)
							    ||'  QUANTITAT VENUDA: '||rpc_rec.venda);
			--Incrementam la posició del rànquing
			POSICIO:=POSICIO+1;
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END RANQUINGMESVENEDOR;
	
	
	----------------------
	--RANQUINGMESREVISIO--
	----------------------
	PROCEDURE RANQUINGMESREVISIO (PDATAINI NUMERIC, PDATAFI NUMERIC) IS
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
			select dataany,mesany,sum(numrevisio) num from ddata d, agrupatrevisio a 
				where d.iddata between to_number(PDATAINI||'01') and to_number(PDATAFI||31)
				and d.iddata=a.iddata
				group by dataany,mesany;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;	
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.RANQUINGMESREVISIO','P1:'||PDATAINI||' P2:'||PDATAFI);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);
	    --Mostram les dades per pantalla
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('ANY: '||rpc_rec.dataany||chr(9)||'  MES: '|| rpc_rec.mesany||chr(9)
						     ||'  NUM REVISIONS: '||rpc_rec.num);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END RANQUINGMESREVISIO;
	
	
	------------------------
	--RANQUINGMESREPARACIO--
	------------------------
	PROCEDURE RANQUINGMESREPARACIO (PDATAINI NUMERIC, PDATAFI NUMERIC) IS
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
			select dataany,mesany,sum(numreparacio) num from ddata d, agrupatreparacio a 
				where d.iddata between to_number(PDATAINI||'01') and to_number(PDATAFI||31)
				and d.iddata=a.iddata
				group by dataany,mesany;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.RANQUINGMESREPARACIO','P1:'||PDATAINI||' P2:'||PDATAFI);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);
	    --Mostram les dades per pantalla
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('ANY: '||rpc_rec.dataany||chr(9)||'  MES: '|| rpc_rec.mesany||chr(9)
						     ||'  NUM REPARACIONS: '||rpc_rec.num);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END RANQUINGMESREPARACIO;
	
	
	--------------------
	--RANQUINGMESVENDA--
	--------------------
	PROCEDURE RANQUINGMESVENDA (PDATAINI NUMERIC, PDATAFI NUMERIC) IS
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
			select dataany,mesany,sum(numvendes) num from ddata d, agrupatvenda a 
				where d.iddata between to_number(PDATAINI||'01') and to_number(PDATAFI||31)
				and d.iddata=a.iddata
				group by dataany,mesany;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;	
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.RANQUINGMESVENDA','P1:'||PDATAINI||' P2:'||PDATAFI);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);
	    --Mostram les dades per pantalla
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('ANY: '||rpc_rec.dataany||chr(9)||'  MES: '|| rpc_rec.mesany||chr(9)
						     ||'  NUM REVISIONS: '||rpc_rec.num);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END RANQUINGMESVENDA;
	
	
	-------------------------
	--RANQUINGDESPESACLIENT--
	-------------------------
	PROCEDURE RANQUINGDESPESACLIENT IS
		--Utilitzam el cursor RPC per extreure de les taules les dades que tenen com a data el període passat per paràmetre							
		CURSOR RPC IS
			SELECT idClient,sum(sumvendes) venda FROM despesaClient dC
				group by idClient
				ORDER BY 2 DESC;
				
		--Utilitzam la variable POSICIO per poder controlar fàcilment quina posició ocupa dins del rànquing
		POSICIO INTEGER:=1;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.RANQUINGDESPESACLIENT','NO APLICA');
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);
	    --Mostram les dades per pantalla
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('POSICIO: '||POSICIO||chr(9)||'  IDCLIENT: '|| rpc_rec.idClient||chr(9)
							    ||'  DESPESA: '||rpc_rec.venda);
			--Incrementam la posició del rànquing
			POSICIO:=POSICIO+1;
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END RANQUINGDESPESACLIENT;
	
END PKG_DWH;
/

spool off;
