/*
	NOM PAQUET: PKG_RRHH
	
	FUNCIONALITAT: Proporciona les operacions necessàries per treballar en el sistema amb VENEDORS i MECANICS
	               
*/

spool 4_5_Implementacio_PkgRRHH.log;
-------------
--CAPÇALERA--
-------------
create or replace package pkg_RRHH is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(8) := 'PKG_RRHH';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	--Permet donar d'alta un nou venedor en el sistema.
	PROCEDURE ALTA_VENEDOR (
		PidVenedor numeric,
		PnomVenedor Varchar2,
		Pllinatge1Venedor Varchar2,
		Pllinatge2Venedor Varchar2,
		PtelefonVenedor numeric,
		PdataAlta date 
	);
	
	--Permet donar de baixa un venedor del sistema
	PROCEDURE BAIXA_VENEDOR ( 
		PidVenedor numeric
	);
	
	--Permet modificar les dades d'un venedor existent en el sistema
	PROCEDURE MODIFICA_VENEDOR (
		PidVenedor numeric,
		PnomVenedor Varchar2,
		Pllinatge1Venedor Varchar2,
		Pllinatge2Venedor Varchar2,
		PtelefonVenedor numeric,
		PdataAlta date 
	);
	
	--Permet donar d'alta un mecanic en el sistema
	PROCEDURE ALTA_MECANIC (
		PidMecanic numeric,
		PnomMecanic Varchar2,
		Pllinatge1Mecanic Varchar2,
		Pllinatge2Mecanic Varchar2,
		PidCatMecanic numeric
	);
	
	--Permet donar de baixa un mecanic del sistema
	PROCEDURE BAIXA_MECANIC ( 
		PidMecanic numeric
	);
	
	--Permet modificar les dades d'un mecànic existent en el sistema
	PROCEDURE MODIFICA_MECANIC (
		PidMecanic numeric,
		PnomMecanic Varchar2,
		Pllinatge1Mecanic Varchar2,
		Pllinatge2Mecanic Varchar2,
		PidCatMecanic numeric
	);
	
	--Permet donar d'alta una categoria de mecànic en el sistema
	PROCEDURE ALTA_CATEGORIA_MECANIC (
		PidCategoriaMecanic numeric,
		PnomCategoriaMecanic Varchar2,
		PpreuHoraCategoriaMecanic number
	);
	
	--Permet donar de baixa una categoria de mecànic del sistema
	PROCEDURE BAIXA_CATEGORIA_MECANIC ( 
		PidCatMecanic numeric
	);
	
	--Permet modificar les dades d'una categoria de mecànic existent en el sistema
	PROCEDURE MODIFICA_CATEGORIA_MECANIC (
		PidCatMecanic numeric,
		PnomCategoriaMecanic Varchar2,
		PpreuHoraCategoriaMecanic number
	);
	
end pkg_RRHH;
/


--------
--BODY--
--------
create or replace package body pkg_RRHH is

	----------------
	--ALTA_VENEDOR--
	----------------
	PROCEDURE ALTA_VENEDOR (
		PidVenedor numeric,
		PnomVenedor Varchar2,
		Pllinatge1Venedor Varchar2,
		Pllinatge2Venedor Varchar2,
		PtelefonVenedor numeric,
		PdataAlta date 
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_VENEDOR','P1:'||PidVenedor||' P2:'||PnomVenedor||' P3:'||Pllinatge1Venedor||
									     ' P4:'||Pllinatge2Venedor||' P5'||PtelefonVenedor||' P6'||PdataAlta||'');
		
		--Comprovam si el sistema es troba en mode TEST		
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidVenedor = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou venedor en el sistema
				INSERT INTO VENEDOR VALUES(seq_venedor.nextval,PnomVenedor,Pllinatge1Venedor,Pllinatge2Venedor,PtelefonVenedor,PdataAlta);
			ELSE
				--Si l'identificador no és igual a 0, inserim el venedor amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO VENEDOR VALUES(PidVenedor,PnomVenedor,Pllinatge1Venedor,Pllinatge2Venedor,PtelefonVenedor,PdataAlta);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-----------------
	--BAIXA_VENEDOR--
	-----------------
	PROCEDURE BAIXA_VENEDOR ( 
		PidVenedor numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_VENEDOR','P1:'||PidVenedor||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el venedor i confirmam el canvi
			DELETE FROM VENEDOR WHERE idVenedor=PidVenedor;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------------
	--MODIFICA_VENEDOR--
	--------------------
	PROCEDURE MODIFICA_VENEDOR (
		PidVenedor numeric,
		PnomVenedor Varchar2,
		Pllinatge1Venedor Varchar2,
		Pllinatge2Venedor Varchar2,
		PtelefonVenedor numeric,
		PdataAlta date 
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_VENEDOR','P1:'||PidVenedor||' P2:'||PnomVenedor||' P3:'||Pllinatge1Venedor||
									     ' P4:'||Pllinatge2Venedor||' P5'||PtelefonVenedor||' P6'||PdataAlta||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE VENEDOR SET nomVenedor=PnomVenedor,llinatge1Venedor=Pllinatge1Venedor,llinatge2Venedor=Pllinatge2Venedor,
				telefonVenedor=PtelefonVenedor,dataAlta=PdataAlta WHERE idVenedor=PidVenedor;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------
	--ALTA_MECANIC--
	----------------
	PROCEDURE ALTA_MECANIC (
		PidMecanic numeric,
		PnomMecanic Varchar2,
		Pllinatge1Mecanic Varchar2,
		Pllinatge2Mecanic Varchar2,
		PidCatMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_MECANIC','P1:'||PidMecanic||' P2:'||PnomMecanic||' P2:'||Pllinatge1Mecanic||
									     ' P3:'||Pllinatge2Mecanic||' P4:'||PidCatMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0.
			IF PidMecanic = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el nou mecànic en el sistema
				INSERT INTO MECANIC VALUES(seq_mecanic.nextval,PnomMecanic,Pllinatge1Mecanic,Pllinatge2Mecanic,PidCatMecanic);
			ELSE
				--Si l'identificador no és igual a 0, inserim el mecànic amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO MECANIC VALUES(PidMecanic,PnomMecanic,Pllinatge1Mecanic,Pllinatge2Mecanic,PidCatMecanic);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-----------------
	--BAIXA_MECANIC--
	-----------------
	PROCEDURE BAIXA_MECANIC ( 
		PidMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_MECANIC',' P1:'||PidMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el mecànic i confirmam el canvi
			DELETE FROM MECANIC WHERE idMecanic=PidMecanic;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------------
	--MODIFICA_MECANIC--
	---------------------
	PROCEDURE MODIFICA_MECANIC (
		PidMecanic numeric,
		PnomMecanic Varchar2,
		Pllinatge1Mecanic Varchar2,
		Pllinatge2Mecanic Varchar2,
		PidCatMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_MECANIC',' P1:'||PidMecanic||' P2:'||PnomMecanic||' P3:'||Pllinatge1Mecanic||
		                                 ' P4:'||Pllinatge2Mecanic||' P5:'||PidCatMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE MECANIC SET nomMecanic=PnomMecanic,llinatge1Mecanic=Pllinatge1Mecanic,llinatge2Mecanic=Pllinatge2Mecanic,
				idCategoriaMecanic=PidCatMecanic WHERE idMecanic=PidMecanic;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------------------
	--ALTA_CATEGORIA_MECANIC--
	--------------------------
	PROCEDURE ALTA_CATEGORIA_MECANIC (
		PidCategoriaMecanic numeric,
		PnomCategoriaMecanic Varchar2,
		PpreuHoraCategoriaMecanic number
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_CATEGORIA_MECANIC','P1:'||PidCategoriaMecanic||' P2:'||PnomCategoriaMecanic||
													' P2:'||PpreuHoraCategoriaMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST		
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidCategoriaMecanic = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim la nova categoria en el sistema
				INSERT INTO CATEGORIAMECANIC VALUES(seq_catmecanic.nextval,PnomCategoriaMecanic,PpreuHoraCategoriaMecanic);
			ELSE
				--Si l'identificador no és igual a 0, inserim la categoria amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO CATEGORIAMECANIC VALUES(PidCategoriaMecanic,PnomCategoriaMecanic,PpreuHoraCategoriaMecanic);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------------------
	--BAIXA_CATEGORIA_MECANIC--
	---------------------------
	PROCEDURE BAIXA_CATEGORIA_MECANIC ( 
		PidCatMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_CATEGORIA_MECANIC','P1:'||PidCatMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la categoria i confirmam el canvi
			DELETE FROM CATEGORIAMECANIC WHERE idCategoriaMecanic=PidCatMecanic;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------------------
	--MODIFICA_CATEGORIA_MECANIC--
	------------------------------
	PROCEDURE MODIFICA_CATEGORIA_MECANIC (
		PidCatMecanic numeric,
		PnomCategoriaMecanic Varchar2,
		PpreuHoraCategoriaMecanic number
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_CATEGORIA_MECANIC','P1:'||PidCatMecanic||' P2:'||PnomCategoriaMecanic||
								         ' P3:'||PpreuHoraCategoriaMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE CATEGORIAMECANIC SET nomCategoriaMecanic=PnomCategoriaMecanic,preuHoraCategoriaMecanic=PpreuHoraCategoriaMecanic 
				WHERE idCategoriaMecanic=PidCatMecanic;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
END;
/

spool off;
