/*
	NOM PAQUET: PKG_TALLER
	
	FUNCIONALITAT: Proporciona les operacions necessàries per treballar amb les revisions i les reparacions
*/

spool 4_6_Implementacio_PkgTaller.log;
-------------
--CAPÇALERA--
-------------
create or replace package pkg_TALLER is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(10) := 'PKG_TALLER';
		
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	--Permet donar d'alta un cotxe del taller en el sistema
	PROCEDURE ALTA_COTXE_TALLER (
		PidCotxe numeric,
		PidClient numeric
	);
	
	--Permet donar de baixa un cotxe del taller en el sistema
	PROCEDURE BAIXA_COTXE_TALLER (
		PidCotxe numeric
	);
	
	--Permet donar d'alta una revisió en el sistema
	PROCEDURE ALTA_REVISIO (
		PidRevisio numeric,
		PidCotxe numeric,
		PdataRevisio date,
		Pmotiu Varchar2,
		PesPressupost Varchar2
	);
	
	--Permet donar de baixa una revisió en el sistema
	PROCEDURE BAIXA_REVISIO (
		PidRevisio numeric
	);
	
	--Permet modificar les dades d'una revisió del sistema
	PROCEDURE MODIFICA_REVISIO (
		PidRevisio numeric,
		PidCotxe numeric,
		PdataRevisio date,
		Pmotiu Varchar2,
		PesPressupost Varchar2
	);
	
	--Permet donar d'alta una reparació en el sistema
	PROCEDURE ALTA_REPARACIO (
		PidReparacio numeric,
		PidCotxe numeric,
		PdataIniciReparacio date,
		PdataFiReparacio date,
		Pmotiu Varchar2,
		PcostPecesReparacio number,
		PesPressupost Varchar2
	);
	
	--Permet donar de baixa una reparació en el sistema
	PROCEDURE BAIXA_REPARACIO (
		PidReparacio numeric
	);
	
	--Permet modificar les dades d'una reparació del sistema
	PROCEDURE MODIFICA_REPARACIO (
		PidReparacio numeric,
		PidCotxe numeric,
		PdataIniciReparacio date,
		PdataFiReparacio date,
		Pmotiu Varchar2,
		PcostPecesReparacio number,
		PesPressupost Varchar2
	);
	
	--Permet donar d'alta el temps de treball realitzat per un mecànic en una reparació
	PROCEDURE ALTA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric,
		PnumHoresReparacio numeric
	);

	--Permet donar de baixa el temps de treball realitat per un mecànic en una reparació
	PROCEDURE BAIXA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric
	);
	
	--Permet modificar el temps de treball (i el preu per hora) d'un mecànic en una reparació
	PROCEDURE MODIFICA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric,
		PnumHoresReparacio numeric,
		PpreuHoresReparacio numeric
	);
	
	--Permet donar d'alta el temps de treball realitzat per un mecànic en una revisió
	PROCEDURE ALTA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric,
		PnumHoresRevisio numeric
	);

	--Permet donar de baixa el temps de treball realitat per un mecànic en una revisió
	PROCEDURE BAIXA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric
	);
	
	--Permet modificar el temps de treball (i el preu per hora) d'un mecànic en una revisió
	PROCEDURE MODIFICA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric,
		PnumHoresRevisio numeric,
		PpreuHoresRevisio numeric
	);
	
end pkg_TALLER;
/

--------
--BODY--
--------
create or replace package body pkg_TALLER is
	
	---------------------
	--ALTA_COTXE_TALLER--
	---------------------
	PROCEDURE ALTA_COTXE_TALLER (
		PidCotxe numeric,
		PidClient numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_COTXE_TALLER','P1:'||PidCotxe||' P2:'||PidClient||'');
		
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, inserim el cotxe (i el seu propietari) dins de la taula COTXESTALLER
			INSERT INTO COTXESTALLER VALUES(PidCotxe,PidClient);
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------------
	--BAIXA_COTXE_TALLER--
	----------------------
	PROCEDURE BAIXA_COTXE_TALLER (
		PidCotxe numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_COTXE_TALLER','P1:'||PidCotxe||'');
		
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, eliminam el cotxe dins de la taula cotxestaller
			DELETE FROM COTXESTALLER WHERE idCotxe=PidCotxe;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	----------------
	--ALTA_REVISIO--
	----------------
	PROCEDURE ALTA_REVISIO (
		PidRevisio numeric,
		PidCotxe numeric,
		PdataRevisio date,
		Pmotiu Varchar2,
		PesPressupost Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_REVISIO','P1:'||PidRevisio||' P2:'||PidCotxe||
													' P3:'||PdataRevisio||'  P4:'||Pmotiu||' P5:'||PesPressupost||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidRevisio = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim la nova revisió en el sistema
				INSERT INTO REVISIO VALUES(seq_revisio.nextval,PidCotxe,PdataRevisio,Pmotiu,PesPressupost);
			ELSE
				--Si l'identificador no és igual a 0, inserim la nova revisió amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO REVISIO VALUES(PidRevisio,PidCotxe,PdataRevisio,Pmotiu,PesPressupost);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-----------------
	--BAIXA_REVISIO--
	-----------------
	PROCEDURE BAIXA_REVISIO (
		PidRevisio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;	
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_REVISIO','P1:'||PidRevisio||'');
		
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la reivisió i confirmam el canvi
			DELETE FROM REVISIO WHERE idRevisio=PidRevisio;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------------
	--MODIFICA_REVISIO--
	--------------------
	PROCEDURE MODIFICA_REVISIO (
		PidRevisio numeric,
		PidCotxe numeric,
		PdataRevisio date,
		Pmotiu Varchar2,
		PesPressupost Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_REVISIO','P1:'||PidRevisio||' P2:'||PidCotxe||'P3:'||PdataRevisio||
													' P4:'||Pmotiu||' P5:'||PesPressupost||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE REVISIO SET idCotxe=PidCotxe,dataRevisio=PdataRevisio,motiu=Pmotiu,esPressupost=PesPressupost WHERE idRevisio=PidRevisio;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------
	--ALTA_REVISIO--
	----------------
	PROCEDURE ALTA_REPARACIO (
		PidReparacio numeric,
		PidCotxe numeric,
		PdataIniciReparacio date,
		PdataFiReparacio date,
		Pmotiu Varchar2,
		PcostPecesReparacio number,
		PesPressupost Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_REPARACIO','P1:'||PidReparacio||' P2:'||PidCotxe||' P3:'||PdataIniciReparacio||
													' P4:'||Pmotiu||' P5:'||PdataFiReparacio||' P6:'||PcostPecesReparacio||' P7:'||PesPressupost||'');
		
		--Comprovam si el sistema es troba en mode TEST
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).		
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidReparacio = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim la nova reparacio en el sistema
				INSERT INTO REPARACIO VALUES(seq_reparacio.nextval,PidCotxe,PdataIniciReparacio,PdataFiReparacio,Pmotiu,PcostPecesReparacio,PesPressupost);
			ELSE
				--Si l'identificador no és igual a 0, inserim la reparacio amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO REPARACIO VALUES(PidReparacio,PidCotxe,PdataIniciReparacio,PdataFiReparacio,Pmotiu,PcostPecesReparacio,PesPressupost);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-----------------
	--BAIXA_REVISIO--
	-----------------
	PROCEDURE BAIXA_REPARACIO (
		PidReparacio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_REPARACIO','P1:'||PidReparacio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la reparació i confirmam el canvi
			DELETE FROM REPARACIO WHERE idReparacio=PidReparacio;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------------
	--MODIFICA_REVISIO--
	--------------------
	PROCEDURE MODIFICA_REPARACIO (
		PidReparacio numeric,
		PidCotxe numeric,
		PdataIniciReparacio date,
		PdataFiReparacio date,
		Pmotiu Varchar2,
		PcostPecesReparacio number,
		PesPressupost Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_REPARACIO','P1:'||PidReparacio||' P2:'||PidCotxe||' P3:'||PdataIniciReparacio||
													' P4:'||Pmotiu||' P5:'||PdataFiReparacio||' P6:'||PcostPecesReparacio||' P7:'||PesPressupost||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE REPARACIO SET idCotxe=PidCotxe,dataIniciReparacio=PdataIniciReparacio,dataFiReparacio=PdataFiReparacio, 
							costPecesReparacio=PcostPecesReparacio,esPressupost=PesPressupost,motiu=Pmotiu WHERE idReparacio=PidReparacio;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------------------
	--ALTA_REPARACIO_MECANIC--
	--------------------------
	PROCEDURE ALTA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric,
		PnumHoresReparacio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
		
		--Variable que s'utilitza per desar el preu per hora corresponent a la categoria del mecànic
		TpreuHoresReparacio numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_REPARACIO_MECANIC','P1:'||PidReparacio||' P2:'||PidMecanic||' P3:'||PnumHoresReparacio||'');

		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE		
			--Cercam el preu per hora corresponent a la categoria del mecànic passat per paràmetre
			SELECT preuHoraCategoriaMecanic INTO TpreuHoresReparacio
				FROM CATEGORIAMECANIC CT, MECANIC M WHERE M.idMecanic=PidMecanic 
				AND CT.idCategoriaMecanic=M.idCategoriaMecanic;
				
			--Inserim el cost en la taula HAREPARAT i confirmam el canvi
			INSERT INTO HAREPARAT VALUES(PidReparacio,PidMecanic,PnumHoresReparacio,TpreuHoresReparacio);
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------------------
	--BAIXA_REPARACIO_MECANIC--
	---------------------------
	PROCEDURE BAIXA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_REPARACIO_MECANIC','P1:'||PidReparacio||' P2:'||PidMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el registre i confirmam el canvi
			DELETE FROM HAREPARAT WHERE idReparacio=PidReparacio AND idMecanic=PidMecanic;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------------------
	--MODIFICA_REPARACIO_MECANIC--
	------------------------------
	PROCEDURE MODIFICA_REPARACIO_MECANIC (
		PidReparacio numeric,
		PidMecanic numeric,
		PnumHoresReparacio numeric,
		PpreuHoresReparacio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_REPARACIO_MECANIC','P1:'||PidReparacio||' P2:'||PidMecanic||' P3:'||PnumHoresReparacio||
													' P4:'||PpreuHoresReparacio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE HAREPARAT SET numHoresReparacio=PnumHoresReparacio,preuHoresReparacio=PpreuHoresReparacio
								WHERE idReparacio=PidReparacio AND idMecanic=PidMecanic;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------------
	--ALTA_REVISIO_MECANIC--
	------------------------
	PROCEDURE ALTA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric,
		PnumHoresRevisio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
		--Variable que s'utilitza per desar el preu per hora corresponent a la categoria del mecànic
		TpreuHoresRevisio numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_REVISIO_MECANIC','P1:'||PidRevisio||' P2:'||PidMecanic||' P3:'||PnumHoresRevisio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Cercam el preu per hora corresponent a la categoria del mecànic passat per paràmetre
			SELECT preuHoraCategoriaMecanic INTO TpreuHoresRevisio
				FROM CATEGORIAMECANIC CT, MECANIC M WHERE M.idMecanic=PidMecanic 
				AND CT.idCategoriaMecanic=M.idCategoriaMecanic;
		
			--Inserim el registre dins de la taula HAREVISAT i confirmam el canvi
			INSERT INTO HAREVISAT VALUES(PidRevisio,PidMecanic,PnumHoresRevisio,TpreuHoresRevisio);
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	-------------------------
	--BAIXA_REVISIO_MECANIC--
	-------------------------
	PROCEDURE BAIXA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_REVISIO_MECANIC','P1:'||PidRevisio||' P2:'||PidMecanic||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el registre corresponent
			DELETE FROM HAREVISAT WHERE idRevisio=PidRevisio AND idMecanic=PidMecanic;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------------------
	--MODIFICA_REVISIO_MECANIC--
	----------------------------
	PROCEDURE MODIFICA_REVISIO_MECANIC (
		PidRevisio numeric,
		PidMecanic numeric,
		PnumHoresRevisio numeric,
		PpreuHoresRevisio numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_REVISIO_MECANIC','P1:'||PidRevisio||' P2:'||PidMecanic||' P3:'||PnumHoresRevisio||
													' P4:'||PpreuHoresRevisio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE HAREVISAT SET numHoresRevisio=PnumHoresRevisio,preuHoresRevisio=PpreuHoresRevisio
								WHERE idRevisio=PidRevisio AND idMecanic=PidMecanic;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
end pkg_TALLER;
/

spool off;
