/*
	NOM PAQUET: PKG_VENDES
	
	FUNCIONALITAT: Proporciona les operacions necessàries per treballar amb les vendes i les reserves

*/

spool 4_7_Implementacio_PkgVendes.log
-------------
--CAPÇALERA--
-------------
create or replace package pkg_Vendes is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(10) := 'PKG_VENDES';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	--Si és així no es realitzarà cap modificació durant l'execució del procediment o funció.
	modeTest boolean;
	
	--Permet afegir un cotxe dins de l'stock de vendes
	PROCEDURE ALTA_COTXE_VENDA (
		PidCotxe numeric,
		PdataEntradaCotxe date,
		PnouSegonaMa varchar2, -- 'N' de Nou o 'S' de Segonama
		PpreuSegonaMa number,
		PdataPrimeraMatriculacio date
	);
	
	--Permet eliminar de l'stock de vendes un cotxe
	PROCEDURE BAIXA_COTXE_VENDA (
		PidCotxe numeric
	);
	
	--Permet modificar els valors d'un cotxe en venda
	PROCEDURE MODIFICA_COTXE_VENDA (
		PidCotxe numeric,
		PdataEntradaCotxe date,
		PnouSegonaMa varchar2, -- 'N' de Nou o 'S' de Segonama
		PpreuSegonaMa number,
		PdataPrimeraMatriculacio date
	);
	
	--Permet donar d'alta una venda
	PROCEDURE ALTA_VENDA (
		PidVenda numeric,
		PidCotxe numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuVenda number,
		PdataVenda date,
		PidReserva numeric
	);
	
	--Permet donar de baixa una venda
	PROCEDURE BAIXA_VENDA (
		PidVenda numeric
	);
	
	--Permet modificar els atributs d'una venda
	PROCEDURE MODIFICA_VENDA (
		PidVenda numeric,
		PidCotxe numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuVenda number,
		PdataVenda date,
		PidReserva numeric
	);
	
	--Permet donar d'alta una reserva
	PROCEDURE ALTA_RESERVA (
		PidReserva numeric,
		PidModel numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuReserva number,
		PdataReserva date,
		PdataArribadaVehicle date
	);
	
	--Permet donar de baixa una reserva
	PROCEDURE BAIXA_RESERVA (
		PidReserva numeric
	);
	
	--Permet modificar els atributs d'una reserva
	PROCEDURE MODIFICA_RESERVA (
		PidReserva numeric,
		PidModel numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuReserva number,
		PdataReserva date,
		PdataArribadaVehicle date
	);
	
	--Permet associar un EXTRA a una RESERVA
	PROCEDURE ALTA_EXTRA_RESERVA (
		PidReserva numeric,
		PidExtra numeric
	);
	
	--Permet desassociar un EXTRA a una RESERVA
	PROCEDURE BAIXA_EXTRA_RESERVA (
		PidReserva numeric,
		PidExtra numeric
	);

	--Permet passar una RESERVA a una VENDA
	PROCEDURE PASSA_RESERVA_VENDA (
		PidReserva numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2
	);
		
end pkg_VENDES;
/


--------
--BODY--
--------
create or replace package body pkg_VENDES is

	--------------------
	--ALTA_COTXE_VENDA--
	--------------------
	PROCEDURE ALTA_COTXE_VENDA (
		PidCotxe numeric,
		PdataEntradaCotxe date,
		PnouSegonaMa varchar2, -- 'N' de Nou o 'S' de Segonama
		PpreuSegonaMa number,
		PdataPrimeraMatriculacio date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_COTXE_VENDA','P1:'||PidCotxe||' P2:'||PdataEntradaCotxe||'P3:'||PnouSegonaMa||
													' P4:'||PpreuSegonaMa||' P5:'||PdataPrimeraMatriculacio||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, inserim el registre dins de COTXESVENDA
			INSERT INTO COTXESVENDA VALUES(PidCotxe,PdataEntradaCotxe,PnouSegonaMa,PpreuSegonaMa,PdataPrimeraMatriculacio);
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------------
	--BAIXA_COTXE_VENDA--
	---------------------
	PROCEDURE BAIXA_COTXE_VENDA ( 
		PidCotxe numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_COTXE_VENDA','P1:'||PidCotxe||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el registre corresponent i desam el canvi
			DELETE FROM COTXESVENDA WHERE idCotxe=PidCotxe;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	------------------------
	--MODIFICA_COTXE_VENDA--
	------------------------
	PROCEDURE MODIFICA_COTXE_VENDA (
		PidCotxe numeric,
		PdataEntradaCotxe date,
		PnouSegonaMa varchar2, -- 'N' de Nou o 'S' de Segonama
		PpreuSegonaMa number,
		PdataPrimeraMatriculacio date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_COTXE_VENDA','P1:'||PidCotxe||' P2:'||PdataEntradaCotxe||'P3:'||PnouSegonaMa||
													' P4:'||PpreuSegonaMa||' P5:'||PdataPrimeraMatriculacio||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE COTXESVENDA SET dataEntradaCotxe=PdataEntradaCotxe,nouSegonaMa=PnouSegonaMa,preuSegonaMa=PpreuSegonaMa,dataPrimeraMatriculacio=PdataPrimeraMatriculacio 
						WHERE idCotxe=PidCotxe;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	--------------
	--ALTA_VENDA--
	--------------
	PROCEDURE ALTA_VENDA (
		PidVenda numeric,
		PidCotxe numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuVenda number,
		PdataVenda date,
		PidReserva numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_VENDA','P1:'||PidVenda||' P2:'||PidCotxe||'P3:'||PidClient||
													' P4:'||PidVenedor||' P5:'||PpreuVenda||' P6:'||PdataVenda||' P7:'||PidReserva||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0. 
			IF PidVenda = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim el una nova venda en el sistema
				INSERT INTO VENDA VALUES(seq_venda.nextval,PidCotxe,PidClient,PidVenedor,PpreuVenda,PdataVenda,PidReserva);
			ELSE
				--Si l'identificador no és igual a 0, inserim la venda amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO VENDA VALUES(PidVenda,PidCotxe,PidClient,PidVenedor,PpreuVenda,PdataVenda,PidReserva);
			END IF;
			--BORRAM EL COTXE DE DINS COTXESVENDA (el llevam de l'estock)
			PKG_VENDES.BAIXA_COTXE_VENDA(PidCotxe);
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	---------------
	--BAIXA_VENDA--
	---------------
	PROCEDURE BAIXA_VENDA (
		PidVenda numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_VENDA','P1:'||PidVenda||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la venda i confirmam el canvi
			DELETE FROM VENDA WHERE idVenda=PidVenda;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	

	------------------
	--MODIFICA_VENDA--
	------------------
	PROCEDURE MODIFICA_VENDA (
		PidVenda numeric,
		PidCotxe numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuVenda number,
		PdataVenda date,
		PidReserva numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_VENDA','P1:'||PidVenda||' P2:'||PidCotxe||'P3:'||PidClient||
													' P4:'||PidVenedor||' P5:'||PpreuVenda||' P6:'||PdataVenda||' P7:'||PidReserva||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE VENDA SET idCotxe=PidCotxe,idClient=PidClient,idVenedor=PidVenedor,preuVenda=PpreuVenda,
							dataVenda=PdataVenda, idReserva=PidReserva WHERE idVenda=PidVenda;
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	----------------
	--ALTA_RESERVA--
	----------------
	PROCEDURE ALTA_RESERVA (
		PidReserva numeric,
		PidModel numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuReserva number,
		PdataReserva date,
		PdataArribadaVehicle date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_RESERVA','P1:'||PidReserva||' P2:'||PidModel||'P3:'||PidClient||
													' P4:'||PidVenedor||' P5:'||PpreuReserva||' P6:'||PdataReserva||' P7:'||PdataArribadaVehicle||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució normal
			--Comprovam si l'identificador que s'ha passat per paràmetre és igual a 0.
			IF PidReserva = 0 THEN
				--Si l'identificador és igual a 0, n'extreim un de la seqüencia i inserim la nova reserva en el sistema
				INSERT INTO RESERVA VALUES(seq_reserva.nextval,PidModel,PidClient,PidVenedor,PpreuReserva,PdataReserva,PdataArribadaVehicle);
			ELSE
				--Si l'identificador no és igual a 0, inserim la reserva amb l'identificador aportat. Si ja existeix, la Primary Key provocarà un error.
				INSERT INTO RESERVA VALUES(PidReserva,PidModel,PidClient,PidVenedor,PpreuReserva,PdataReserva,PdataArribadaVehicle);
			END IF;
			--Confirmam el nou registre
			COMMIT;
		END IF;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;


	-----------------
	--BAIXA_RESERVA--
	-----------------
	PROCEDURE BAIXA_RESERVA (
		PidReserva numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_RESERVA','P1:'||PidReserva||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram la reserva i confirmam el canvi
			DELETE FROM RESERVA WHERE idReserva=PidReserva;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;


	--------------------
	--MODIFICA_RESERVA--
	--------------------	
	PROCEDURE MODIFICA_RESERVA (
		PidReserva numeric,
		PidModel numeric,
		PidClient numeric,
		PidVenedor numeric,
		PpreuReserva number,
		PdataReserva date,
		PdataArribadaVehicle date
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODIFICA_RESERVA','P1:'||PidReserva||' P2:'||PidModel||'P3:'||PidClient||
													' P4:'||PidVenedor||' P5:'||PpreuReserva||' P6:'||PdataReserva||' P7:'||PdataArribadaVehicle||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, realitzam l'actualització del registre corresponent
			UPDATE RESERVA SET idModel=PidModel,idClient=PidClient,idVenedor=PidVenedor,preuReserva=PpreuReserva,
							dataReserva=PdataReserva, dataArribadaVehicle=PdataArribadaVehicle WHERE idReserva=PidReserva;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;


	----------------------
	--ALTA_EXTRA_RESERVA--
	----------------------	
	PROCEDURE ALTA_EXTRA_RESERVA (
		PidReserva numeric,
		PidExtra numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.ALTA_EXTRA_RESERVA','P1:'||PidReserva||' P2:'||PidExtra||'');
		
		--Comprovam si el sistema es troba en mode TEST							     
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode test, afegim el registre dins la taula EXTRAAFEGITRESERVA
			INSERT INTO EXTRAAFEGITRESERVA VALUES(PidReserva,PidExtra);
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat) 
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;


	-----------------------
	--BAIXA_EXTRA_RESERVA--
	-----------------------	
	PROCEDURE BAIXA_EXTRA_RESERVA (
		PidReserva numeric,
		PidExtra numeric
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.BAIXA_EXTRA_RESERVA','P1:'||PidReserva||' P2:'||PidExtra||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, borram el registre corresponent i confirmam el canvi
			DELETE FROM EXTRAAFEGITRESERVA WHERE idReserva=PidReserva AND idExtra=PidExtra;
			COMMIT;
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
	
	
	-----------------------
	--PASSA_RESERVA_VENDA--
	-----------------------		
	PROCEDURE PASSA_RESERVA_VENDA (
		PidReserva numeric,
		PbastidorCotxe Varchar2,
		PmatriculaCotxe Varchar2
	) IS
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
		
		--Variables temporals
		TidModel numeric;
		TidCotxe numeric;
		TidClient numeric;
		TidVenedor numeric;
		TpreuVenda number;
		
		--Cursor que s'utilitza per tenir les dades dels extres associats a la RESERVA que s'ha passat per paràmetre
		CURSOR CUR_EC IS SELECT idExtra FROM EXTRAAFEGITRESERVA WHERE idReserva = PidReserva;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_Control.REGISTRA_EXECUCIO(NOMPAQUET||'.PASSA_RESERVA_VENDA','P1:'||PidReserva||' P2:'||PbastidorCotxe||' P3:'||PmatriculaCotxe||'');
		
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho desam en el LOG (si el sistema el té activat).
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		ELSE
			--Si no es troba en mode Test, seguim l'execució
			--Seleccionam les dades de la RESERVA i les passam dins les variables temporals
			SELECT idModel,idClient,idVenedor,preuReserva INTO TidModel,TidClient,TidVenedor,TpreuVenda FROM RESERVA WHERE idReserva=PidReserva;
		
			--Donam d'alta el Cotxe (ja que vol dir que s'ha rebut i que s'ha entregat al client)
			pkg_COTXES.ALTA_COTXE(0,PbastidorCotxe,PmatriculaCotxe,TidModel);
			--Cercam l'identificador del cotxe associat al cotxe que acabam de donar d'alta
			SELECT idCotxe INTO TidCotxe FROM COTXE WHERE bastidorCotxe=PbastidorCotxe;
		
			FOR idExtra_rec IN CUR_EC
			LOOP
				--Per a cada EXTRA de la RESERVA, cream el corresponent registre dins de EXTRACOTXE
				INSERT INTO EXTRACOTXE VALUES(idExtra_rec.idExtra,TidCotxe);
			END LOOP;
			COMMIT;
		
			--Donam d'alta la VENDA corresponent amb la data actual
			PKG_VENDES.ALTA_VENDA(0,TidCotxe,TidClient,TidVenedor,TpreuVenda,SYSDATE,PidReserva);
		END IF;
		
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_Control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_Control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
		
END;
/

spool off;
