/*
	NOM PAQUET: PKG_CONSULTES
	
	FUNCIONALITAT: Permet realitzar, mitjançant procediments, consultes al sistema.

*/

spool 4_8_Implementacio_PkgConsultes.log;
-------------
--CAPÇALERA--
-------------
create or replace package PKG_CONSULTES is
	--NOMPAQUET s'utilitza per registrar fàcilment en el LOG el paquet d'on s'ha executat el procediment--
	NOMPAQUET CHAR(13) := 'PKG_CONSULTES';
	
	--modeTest s'utilitza per determinar a l'inici de l'execució si el sistema es troba en mode TEST. 
	modeTest boolean;
	
	--Tipus de dades que permet recuperar les dades d'un cotxe concret
	TYPE COTXE_REC IS RECORD (
		NOMMARCA Varchar2(40),
		NOMMODEL Varchar2(40),
		PREUMODEL Numeric(7,2),
		POTENCIAMODEL Numeric,
		NUMPORTESMODEL Numeric,
		CILINDRADAMODEL Numeric,
		COMBUSTIBLEMODEL Varchar2(40),
		BASTIDORCOTXE Varchar2(10),
		MATRICULACOTXE Varchar2(10)
	);
	
	--Tipus de dades que permet que consultar els detalls d'una revisió i una reparació
	TYPE DETALLCOST_RECORD IS RECORD (
		NOMMECANIC Varchar2(20),
		NOMCATEGORIAMECANIC Varchar2(20),
		NUMHORESREVISIO NUMERIC,
		PREUHORESREVISIO NUMERIC(12,2)
	);
	
	--Donat un DNI, mostra per pantalla les dades del CLIENT
	PROCEDURE CLIENT_DNICIF (
		PdniCifClient Varchar2
	);
	
	--Donat un idClient, mostra per pantalla les dades del CLIENT
	PROCEDURE CLIENT_ID (
		PidClient numeric
	);
	
	--Donat un idVenedor, mostra per pantalla les dades del VENEDOR
	PROCEDURE VENEDOR_ID (
		PidVenedor numeric
	);
	
	--Donat un idModel, mostra per pantalla els detalls del MODEL (amb els extres)
	PROCEDURE MODEL_ID (
		PidModel numeric
	);
	
	--Donat un idCotxe, mostra per pantalla les dades del COTXE (amb extres afegits)
	PROCEDURE COTXE_ID (
		PidCotxe numeric
	);
	
	--Donat un idVenda, mostra per pantalla les dades de la VENDA
	PROCEDURE VENDA_ID (
		PidVenda numeric
	);
	
	--Retorna el llistat de vendes que hi ha en el sistema
	PROCEDURE LLISTAT_VENDES;
	
	--Donat un idRevisio, mostra per pantalla les dades de la REVISIO
	PROCEDURE REVISIO_ID (
		PidRevisio numeric
	);
	
	--Retorna el llistat de revisio que hi ha en el sistema
	PROCEDURE LLISTAT_REVISIONS;
	
	--Donat un idReparacio, mostra per pantalla les dades de la REPARACIO
	PROCEDURE REPARACIO_ID (
		PidReparacio numeric
	);
	
	--Retorna el llistat de reparacio que hi ha en el sistema
	PROCEDURE LLISTAT_REPARACIONS;
	
	--Simula que s'han avisat als clients que han de realitzar revisions entre les dades passades per parámetre
	PROCEDURE AVISAREVISIONSDATA (PdataInici IN DATE, PdataFi IN DATE);
	
end PKG_CONSULTES;
/


--------
--BODY--
--------
create or replace package body PKG_CONSULTES is

	-----------------
	--CLIENT_DNICIF--
	-----------------
	PROCEDURE CLIENT_DNICIF (
		PdniCifClient Varchar2
	) IS
		--Cursor per consultar les dades del client
		CURSOR CC IS SELECT * FROM CLIENT WHERE DNICIFCLIENT=PdniCifClient;
		--Registre on es guarda el resultat del cursor
		rec client%ROWTYPE;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CLIENT_DNICIF','P1:'||PdniCifClient);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);
		--Obrim el cursor
		OPEN CC;
		FETCH CC INTO REC;
		--Revisam si ha retornat registres
		IF CC%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL CLIENT AMB EL DNI '||PdniCifClient);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('ID: '||REC.IDCLIENT);
			DBMS_OUTPUT.PUT_LINE('NOM: '||REC.NOMCLIENT);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 1: '||REC.LLINATGE1CLIENT);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 2: '||REC.LLINATGE2CLIENT);
			DBMS_OUTPUT.PUT_LINE('DATA NAIXAMENT: '||REC.DATANAIXEMENTCLIENT);
			DBMS_OUTPUT.PUT_LINE('SEXE: '||REC.SEXECLIENT);
			DBMS_OUTPUT.PUT_LINE('DNI/CIF: '||REC.DNICIFCLIENT);
			DBMS_OUTPUT.PUT_LINE('TELEFON: '||REC.TELEFONCLIENT);
			DBMS_OUTPUT.PUT_LINE('DATA ALTA: '||REC.DATAALTA);
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL CLIENT AMB EL DNI '||PdniCifClient);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CC;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CLIENT_DNICIF;
	
	
	-------------
	--CLIENT_ID--
	-------------
	PROCEDURE CLIENT_ID (
		PidClient numeric
	) IS
		--Cursor per consultar les dades del client
		CURSOR CC IS SELECT * FROM CLIENT WHERE idClient=PidClient;
		--Registre on es guarda el resultat del cursor
		rec client%ROWTYPE;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.CLIENT_ID','P1:'||PidClient);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el cursor
		OPEN CC;
		FETCH CC INTO REC;
		--Revisam si ha retornat registres
		IF CC%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL CLIENT AMB EL ID '||PidClient);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('ID: '||REC.IDCLIENT);
			DBMS_OUTPUT.PUT_LINE('NOM: '||REC.NOMCLIENT);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 1: '||REC.LLINATGE1CLIENT);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 2: '||REC.LLINATGE2CLIENT);
			DBMS_OUTPUT.PUT_LINE('DATA NAIXAMENT: '||REC.DATANAIXEMENTCLIENT);
			DBMS_OUTPUT.PUT_LINE('SEXE: '||REC.SEXECLIENT);
			DBMS_OUTPUT.PUT_LINE('DNI/CIF: '||REC.DNICIFCLIENT);
			DBMS_OUTPUT.PUT_LINE('TELEFON: '||REC.TELEFONCLIENT);
			DBMS_OUTPUT.PUT_LINE('DATA ALTA: '||REC.DATAALTA);
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL CLIENT AMB EL ID '||PidClient);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CC;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END CLIENT_ID;
	
	--------------
	--VENEDOR_ID--
	--------------
	PROCEDURE VENEDOR_ID (
		PidVenedor numeric
	) IS
		--Cursor per consultar les dades del venedor
		CURSOR CV IS SELECT * FROM VENEDOR WHERE idVenedor=PidVenedor;
		--Registre on es guarda el resultat del cursor
		rec venedor%ROWTYPE;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.VENEDOR_ID','P1:'||PidVenedor);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el cursor
		OPEN CV;
		FETCH CV INTO REC;
		
		--Revisam si ha retornat registres
		IF CV%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL VENEDOR AMB EL ID '||PidVenedor);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('NOM: '||REC.NOMVENEDOR);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 1: '||REC.LLINATGE1VENEDOR);
			DBMS_OUTPUT.PUT_LINE('LLINATGE 2: '||REC.LLINATGE2VENEDOR);
			DBMS_OUTPUT.PUT_LINE('TELEFON: '||REC.TELEFONVENEDOR);
			DBMS_OUTPUT.PUT_LINE('DATA ALTA: '||REC.DATAALTA);
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL VENEDOR AMB EL ID '||PidVenedor);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CV;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END VENEDOR_ID;
	
	--------------
	--MODEL_ID--
	--------------
	PROCEDURE MODEL_ID (
		PidModel numeric
	) IS
		--Cursor per consultar la marca del model
		CURSOR CMM IS SELECT NOMMARCA FROM MARCA MM, MODEL M WHERE M.idModel=PidModel and MM.idMarca=M.idMarca;
		--Cursor per consultar les dades del model
		CURSOR CM IS SELECT * FROM MODEL M WHERE idModel=PidModel;
		--Cursor per obtenir el llistat d'extres del cotxe
		CURSOR CLE IS SELECT EXTRAMODEL.IDEXTRA, EXTRAMODEL.IDMODEL, EXTRAMODEL.PREUEXTRA, EXTRAMODEL.SERIEOPCIO from extra, extramodel, model 
			where extra.idextra=extramodel.idextra 
			AND extramodel.idModel=model.idModel
			AND model.idModel=PidModel;
		--Variables per guardar noms
		TnomMarca Varchar2(20);
		TnomExtra Varchar2(20);
		--Enter per dur el compte dels extres
		NUMEXTRA INTEGER;
		--Registre on es guarda el resultat del cursor
		rec1 model%rowtype;
		--Registre on es guarda el resultat del cursor
		rec2 extramodel%rowtype;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.MODEL_ID','P1:'||PidModel);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el primer cursor
		OPEN CMM;
		FETCH CMM INTO TnomMarca;
		IF CMM%FOUND THEN
			--Cercam les dades del model
			OPEN CM;
			FETCH CM INTO REC1;
			--Revisam si ha retornat registres
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL MODEL AMB EL ID '||PidModel);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('MARCA: '||TnomMarca);
			DBMS_OUTPUT.PUT_LINE('MODEL: '||REC1.NOMMODEL);
			DBMS_OUTPUT.PUT_LINE('PREU: '||REC1.PREUMODEL);
			DBMS_OUTPUT.PUT_LINE('POTENCIA: '||REC1.POTENCIAMODEL);
			DBMS_OUTPUT.PUT_LINE('NUMPORTES: '||REC1.NUMPORTESMODEL);
			DBMS_OUTPUT.PUT_LINE('CILINDRADA: '||REC1.CILINDRADAMODEL);
			DBMS_OUTPUT.PUT_LINE('COMBUSTIBLE: '||REC1.COMBUSTIBLEMODEL);
			NUMEXTRA:=1;
			FOR rec2 IN CLE
			LOOP
				SELECT NOMEXTRA INTO TnomExtra FROM EXTRA WHERE idExtra=REC2.idExtra;
				DBMS_OUTPUT.PUT_LINE('EXTRA'||NUMEXTRA||': '||TnomExtra ||'  SERIE/OPCIO: '||REC2.SERIEOPCIO);
				NUMEXTRA:=NUMEXTRA+1;
			END LOOP;
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL MODEL AMB EL ID '||PidModel);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CM;
		CLOSE CMM;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END MODEL_ID;
	

	--------------
	--COTXE_ID--
	--------------
	PROCEDURE COTXE_ID (
		PidCotxe numeric
	) IS
		--Cursor per consultar les dades del cotxe
		CURSOR CC IS select NOMMARCA,NOMMODEL,PREUMODEL,POTENCIAMODEL,NUMPORTESMODEL,CILINDRADAMODEL,COMBUSTIBLEMODEL,BASTIDORCOTXE,MATRICULACOTXE
			FROM MODEL M, MARCA MC, COTXE C 
			WHERE C.IDCOTXE=PidCotxe
			AND M.IDMARCA=MC.IDMARCA
			AND C.IDMODEL=M.IDMODEL;
		--Cursor per obtenir el llistat d'extres del cotxe
		CURSOR CLE IS SELECT NOMEXTRA 
			from extra, extracotxe 
			where extra.idextra=extracotxe.idextra and idcotxe=PidCotxe;
		--Senser per dur el compte dels extres
		NUMEXTRA INTEGER;
		--Variable per 
		STREXTRA VARCHAR2(150):='';
		--Registre on es guarda el resultat del cursor
		rec cotxe_rec;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.COTXE_ID','P1:'||PidCotxe);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el cursor
		OPEN CC;
		FETCH CC INTO REC;
		--Revisam si ha retornat registres
		IF CC%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DEL COTXE AMB EL ID '||PidCotxe);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('MARCA: '||REC.NOMMARCA);
			DBMS_OUTPUT.PUT_LINE('MODEL: '||REC.NOMMODEL);
			DBMS_OUTPUT.PUT_LINE('PREU: '||REC.PREUMODEL);
			DBMS_OUTPUT.PUT_LINE('POTENCIA: '||REC.POTENCIAMODEL);
			DBMS_OUTPUT.PUT_LINE('NUMPORTES: '||REC.NUMPORTESMODEL);
			DBMS_OUTPUT.PUT_LINE('CILINDRADA: '||REC.CILINDRADAMODEL);
			DBMS_OUTPUT.PUT_LINE('COMBUSTIBLE: '||REC.COMBUSTIBLEMODEL);
			DBMS_OUTPUT.PUT_LINE('BASTIDORCOTXE: '||REC.BASTIDORCOTXE);
			DBMS_OUTPUT.PUT_LINE('MATRICULACOTXE: '||REC.MATRICULACOTXE);
			NUMEXTRA:=1;
			FOR rec IN CLE
			LOOP
				STREXTRA:=STREXTRA||'  EXTRA'||NUMEXTRA||':'||rec.NOMEXTRA;
				NUMEXTRA:=NUMEXTRA+1;
			END LOOP;
			DBMS_OUTPUT.PUT_LINE(STREXTRA);
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DEL COTXE AMB EL ID '||PidCotxe);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CC;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END COTXE_ID;
	
	--------------
	--VENDA_ID--
	--------------
	PROCEDURE VENDA_ID (
		PidVenda numeric
	) IS
		--Cursor per consultar les dades de la venda
		CURSOR CV IS SELECT * FROM VENDA WHERE idVenda=PidVenda;
		--Registre on es guarda el resultat del cursor
		rec venda%ROWTYPE;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.VENDA_ID','P1:'||PidVenda);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el cursor
		OPEN CV;
		FETCH CV INTO REC;
		--Revisam si ha retornat registres
		IF CV%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DE LA VENDA AMB EL ID '||PidVenda);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.NEW_LINE;
			DBMS_OUTPUT.PUT_LINE('IDVENDA: '||REC.IDVENDA);
			DBMS_OUTPUT.PUT_LINE('IDCOTXE: '||REC.IDCOTXE);
			DBMS_OUTPUT.PUT_LINE('IDCLIENT: '||REC.IDCLIENT);
			DBMS_OUTPUT.PUT_LINE('IDVENEDOR: '||REC.IDVENEDOR);
			DBMS_OUTPUT.PUT_LINE('PREUVENDA: '||REC.PREUVENDA);
			DBMS_OUTPUT.PUT_LINE('DATAVENDA: '||REC.DATAVENDA);
			DBMS_OUTPUT.PUT_LINE('IDRESERVA: '||REC.IDRESERVA);
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DADES DE LA VEND AMB EL ID '||PidVenda);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;
		--Tancam el cursor
		CLOSE CV;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END VENDA_ID;


	-----------------
	--LLISTAT_VENDES-
	-----------------
	PROCEDURE LLISTAT_VENDES IS
		--Cursor per consultar les dades de totes les vendes
		CURSOR CLV IS SELECT IDVENDA FROM VENDA;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LLISTAT_VENDES','NO APLICA');
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		DBMS_OUTPUT.PUT_LINE('******************');
		DBMS_OUTPUT.PUT_LINE('LLISTAT DE VENDES');
		DBMS_OUTPUT.PUT_LINE('******************');
		--Per a cada venda, imprimim el detall
		FOR TidVenda IN CLV
		LOOP
			PKG_CONSULTES.VENDA_ID(TidVenda.idVenda);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END LLISTAT_VENDES;
	
	
	--------------
	--REVISIO_ID--
	--------------
	PROCEDURE REVISIO_ID (
		PidRevisio numeric
	) IS
		--Cursor per consultar el motiu de la revisió
		CURSOR CR1 IS SELECT * FROM REVISIO WHERE idRevisio=PidRevisio;
		--Cursor per consultar els costs de la revisio
		CURSOR CR2 IS SELECT SUM(numHoresRevisio*preuHoresRevisio) cost FROM REVISIO RV, HAREVISAT HRV
				WHERE HRV.idRevisio=RV.idRevisio
				and HRV.idRevisio=PidRevisio
				GROUP BY Motiu,HRV.idRevisio,esPressupost;
		--Cursor per consultar els costs de mà d'obra
		CURSOR CR3 IS SELECT NOMMECANIC,NOMCATEGORIAMECANIC,NUMHORESREVISIO HORES,PREUHORESREVISIO PREU
			FROM revisio r, harevisat hrv, mecanic m, categoriamecanic cm
			WHERE cm.idcategoriamecanic=m.idcategoriamecanic
			AND hrv.idmecanic=m.idmecanic
			AND hrv.idrevisio=r.idrevisio
			AND r.idrevisio=PidRevisio;
		--Registre per guardar les dades del primer cursor
		rec1 revisio%ROWTYPE;
		--Registre per guardar les dades del segon cursor
		rec2 numeric;
		--Registre per guardar les dades del tercer cursor
		rec3 DETALLCOST_RECORD;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.REVISIO_ID','P1:'||PidRevisio);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el primer cursor per obtenir el motiu
		OPEN CR1;
		FETCH CR1 INTO rec1;
		--Revisam si ha retornat registres
		IF CR1%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DE LA REVISIO AMB EL ID '||PidRevisio);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('IDCOTXE: '||rec1.IDCOTXE);
			DBMS_OUTPUT.PUT_LINE('DATA: '||rec1.DATAREVISIO);
			DBMS_OUTPUT.PUT_LINE('MOTIU: '||rec1.MOTIU);
			DBMS_OUTPUT.PUT_LINE('ES PRESSUPOST: '||rec1.ESPRESSUPOST);
			--Obrim el segon cursor per obtenir els costs
			OPEN CR2;
			FETCH CR2 INTO REC2;
			--Revisam si ha retornat registres
			IF CR2%FOUND THEN
				DBMS_OUTPUT.PUT_LINE('COST: '||REC2);
				FOR rec3 IN CR3
				LOOP
					DBMS_OUTPUT.PUT_LINE('NOM: '||REC3.NOMMECANIC|| chr(9) ||'CATEGORIA: '||REC3.NOMCATEGORIAMECANIC|| chr(9) 
									   ||'NUM HORES REVISIO: '||REC3.HORES|| chr(9) ||'PREU HORES REVISIO: '||REC3.PREU|| chr(9));
				END LOOP;
			ELSE
				DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DETALLS DELS COSTOS DE LA REVISIO');
			END IF;
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DETALLS DE COSTOS DE LA REVISIO AMB EL ID '||PidRevisio);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;			
		--Tancam els cursors
		CLOSE CR1;
		CLOSE CR2;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END REVISIO_ID;


	--------------------
	--LLISTAT_REVISIONS-
	--------------------
	PROCEDURE LLISTAT_REVISIONS IS
		--Cursor per consultar les dades de totes les revisions
		CURSOR CLR IS SELECT IDREVISIO FROM REVISIO;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LLISTAT_REVISIONS','NO APLICA');
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		DBMS_OUTPUT.PUT_LINE('******************');
		DBMS_OUTPUT.PUT_LINE('LLISTAT DE REVISIONS');
		DBMS_OUTPUT.PUT_LINE('******************');
		--Per a cada venda, imprimim el detall
		FOR TidRevisio IN CLR
		LOOP
			PKG_CONSULTES.REVISIO_ID(TidRevisio.idRevisio);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END LLISTAT_REVISIONS;
	
	
	----------------
	--REPARACIO_ID--
	----------------
	PROCEDURE REPARACIO_ID (
		PidReparacio numeric
	) IS
		--Cursor per consultar el motiu de la reparació
		CURSOR CR1 IS SELECT * FROM REPARACIO WHERE idReparacio=PidReparacio;
		--Cursor per consultar els costs de la reparació
		CURSOR CR2 IS SELECT SUM(numHoresReparacio*preuHoresReparacio)+costPecesReparacio cost FROM REPARACIO RP, HAREPARAT HRP
				WHERE HRP.idReparacio=RP.idReparacio
				and HRP.idReparacio=PidReparacio
				GROUP BY HRP.idReparacio,esPressupost,costPecesReparacio;
		--Cursor per consultar els costs de mà d'obra
		CURSOR CR3 IS SELECT NOMMECANIC,NOMCATEGORIAMECANIC,NUMHORESREPARACIO HORES,PREUHORESREPARACIO PREU
			FROM reparacio r, hareparat hrp, mecanic m, categoriamecanic cm
			WHERE cm.idcategoriamecanic=m.idcategoriamecanic
			AND hrp.idmecanic=m.idmecanic
			AND hrp.idReparacio=r.idReparacio
			AND r.idReparacio=PidReparacio;
		--Registre per guardar les dades del primer cursor
		rec1 reparacio%ROWTYPE;
		--Registre per guardar les dades del segon cursor
		rec2 numeric;
		--Registre per guardar les dades del tercer cursor
		rec3 DETALLCOST_RECORD;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.REPARACIO_ID','P1:'||PidReparacio);
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		--Obrim el primer cursor per obtenir el motiu
		OPEN CR1;
		FETCH CR1 INTO rec1;
		--Revisam si ha retornat registres
		IF CR1%FOUND THEN
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DADES DE LA REPARACIO AMB EL ID '||PidReparacio);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('IDCOTXE: '||rec1.IDCOTXE);
			DBMS_OUTPUT.PUT_LINE('DATA INICI REPARACIO: '||rec1.DATAINICIREPARACIO);
			DBMS_OUTPUT.PUT_LINE('DATA FI REPARACIO: '||rec1.DATAFIREPARACIO);
			DBMS_OUTPUT.PUT_LINE('MOTIU: '||rec1.MOTIU);
			DBMS_OUTPUT.PUT_LINE('COST PECES REPARACIO: '||rec1.COSTPECESREPARACIO);
			DBMS_OUTPUT.PUT_LINE('ES PRESSUPOST: '||rec1.ESPRESSUPOST);
			--Obrim el segon cursor per obtenir els costs
			OPEN CR2;
			FETCH CR2 INTO REC2;
			--Revisam si ha retornat registres
			IF CR2%FOUND THEN
				DBMS_OUTPUT.PUT_LINE('COST TOTAL: '||REC2);
				FOR rec3 IN CR3
				LOOP
					DBMS_OUTPUT.PUT_LINE('NOM: '||REC3.NOMMECANIC|| chr(9) ||'CATEGORIA: '||REC3.NOMCATEGORIAMECANIC|| chr(9) 
									   ||'NUM HORES REVISIO: '||REC3.HORES|| chr(9) ||'PREU HORES REVISIO: '||REC3.PREU|| chr(9));
				END LOOP;
			ELSE
				DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DETALLS DELS COSTOS DE LA REPARACIO');
			END IF;
		ELSE
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.PUT_LINE('NO S''HAN TROBAT DETALLS DE COSTOS DE LA REPARACIO AMB EL ID '||PidReparacio);
			DBMS_OUTPUT.PUT_LINE('****************************************************');
			DBMS_OUTPUT.NEW_LINE;
		END IF;			
		--Tancam els cursors
		CLOSE CR1;
		CLOSE CR2;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END REPARACIO_ID;


	----------------------
	--LLISTAT_REPARACIONS-
	----------------------
	PROCEDURE LLISTAT_REPARACIONS IS
		--Cursor per consultar les dades de totes les reparació
		CURSOR CLR IS SELECT IDREPARACIO FROM REPARACIO;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.LLISTAT_REPARACIONS','NO APLICA');
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;
		--Ampliam el buffer del output
		DBMS_OUTPUT.ENABLE(1000000);	
		DBMS_OUTPUT.PUT_LINE('******************');
		DBMS_OUTPUT.PUT_LINE('LLISTAT DE REPARACIONS');
		DBMS_OUTPUT.PUT_LINE('******************');
		--Per a cada venda, imprimim el detall
		FOR TidReparacio IN CLR
		LOOP
			PKG_CONSULTES.REPARACIO_ID(TidReparacio.idReparacio);
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END LLISTAT_REPARACIONS;
	
	----------------------
	--AVISAREVISIONSDATA--
	----------------------
	PROCEDURE AVISAREVISIONSDATA (PdataInici IN DATE, PdataFi IN DATE) IS
		--Cursor que s'utilitza per determinar la data de revisió, el nom del client i el telèfon del client de les revisions
		--compreses en el període passat per paràmetre.
		CURSOR RPC IS
			select idRevisio,dataRevisio,nomClient,telefonClient 
			FROM REVISIO R, COTXESTALLER CT, CLIENT C
			WHERE R.idCotxe = CT.idCotxe
			AND CT.idClient = C.idClient
			AND dataRevisio BETWEEN PdataInici AND PdataFi
			ORDER BY 1 ASC;
		--idLog s'utilitza per guardar l'identificador de LOG que ens retorna el registrar una nova execució
		idLog numeric;
	BEGIN				
		--Registram l'execucio (amb els paràmetres corresponents) i desam l'identificador de LOG que ens retorna  (si el sistema el té activat)--
		idLog:=pkg_control.REGISTRA_EXECUCIO(NOMPAQUET||'.AVISAREVISIONSDATA','P1:'||PdataInici||'P2:'||PdataFi);	
		--Comprovam si el sistema es troba en mode TEST	
		modeTest:=PKG_CONTROL.MODE_TEST();
		--Imprimim les dades de les revisions
		IF modeTest THEN
			--Si es troba en mode Test, ho posam en el log però seguim amb l'execució normal, ja que les consultes no modifiquen res
			PKG_CONTROL.TEST_EXECUCIO(idLog);
		END IF;													
		FOR rpc_rec IN RPC
		LOOP
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('AVISANT PER REVISIÓ AMB ID '||rpc_rec.idRevisio);
			DBMS_OUTPUT.PUT_LINE('**************************************');
			DBMS_OUTPUT.PUT_LINE('DATA REVISIO: '||rpc_rec.dataRevisio);
			DBMS_OUTPUT.PUT_LINE('NOM CLIENT: '||rpc_rec.nomClient);
			DBMS_OUTPUT.PUT_LINE('TELEFON CLIENT: '||rpc_rec.telefonClient);
			--Cada registre del RPC l'inserim dins de la varibale REC
		END LOOP;
		--Registram el FI de l'execució en la Taula de LOG (si el sistema el té activat)
		pkg_control.FI_EXECUCIO(idLog);
	EXCEPTION 
		--En el cas de produïr-se una excepció, aquesta es registra en el LOG (si el sistema el té activat)
		WHEN OTHERS THEN
			pkg_control.ERROR_EXECUCIO(idLog,'ERROR '||SUBSTR(SQLERRM, 1, 200));
	END;
END;
/

spool off;
